#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ chrono_help.rb ver. 1.0.  28-Sep-11
#~ The script, which enables "Help" command in Chronolux sub-menu and within LSS Chronolux Toolbar
#~ and within "LSS Chronolux Toolbar"

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# This class enables "Help" command in Chronolux sub-menu and within LSS Chronolux Toolbar

class Lss_Chronolux_Help
  
  # Get paths for help files (for index file and for help files with Chronolux tools descriptions)
  def initialize
    @resource_dir = File.dirname(Sketchup.get_resource_path("chronolux.strings"))
    @index_path="#{@resource_dir}/chronolux/index.html"
    @add_tst_pt_path="#{@resource_dir}/chronolux/add_tst_pt.html"
    @pnts_list_path="#{@resource_dir}/chronolux/list_tst_pts.html"
    @test_face_path="#{@resource_dir}/chronolux/pick_tst_face.html"
  end
  
  #Add help command (it's a bit context sensitive: if one of Chronolux tools is active, then appropriate help
  # topic opens in default browser
  def add_help_cmd
    chrono_help_cmd=UI::Command.new($chronoluxStrings.GetString("Help")){
      model=Sketchup.active_model
      lss_tool_name=model.get_attribute("LSS Chronolux Settings", "lss_tool_name")
      case lss_tool_name
        when "Chronolux Add Test Pnt Tool"
        UI.openURL(@add_tst_pt_path)
        when "Chronolux List Test Points Tool"
        UI.openURL(@pnts_list_path)
        when "Chronolux Pick Test Face Tool"
        UI.openURL(@test_face_path)
        else
        UI.openURL(@index_path)
      end
    }
    chrono_help_cmd.small_icon = "./tb_icons/chrono_help_16.png"
    chrono_help_cmd.large_icon = "./tb_icons/chrono_help_24.png"
    chrono_help_cmd.tooltip = $chronoluxStrings.GetString("Click to view help index or help on active tool")
    $chronoluxToolbar.add_separator
    $chronoluxToolbar.add_item(chrono_help_cmd)
    $chronoluxMenu.add_separator
    $chronoluxMenu.add_item(chrono_help_cmd)
  end
  
  # Add "Visit Program Website..." command
  def add_visit_site_cmd
	chrono_visit_cmd=UI::Command.new($chronoluxStrings.GetString("Visit Program Website...")){
		UI.openURL("https://sites.google.com/site/lssoft2011/")
	}
	chrono_visit_cmd.small_icon = "./tb_icons/chrono_www_16.png"
	chrono_visit_cmd.large_icon = "./tb_icons/chrono_www_24.png"
	chrono_visit_cmd.tooltip = $chronoluxStrings.GetString("Click to visit Chronolux for Sketchup website")
	$chronoluxToolbar.add_item(chrono_visit_cmd)
	$chronoluxMenu.add_item(chrono_visit_cmd)  
  end
  
end #class Lss_Chronolux_Help

if( not file_loaded?("chrono_help.rb") )
  lss_chrono_help=Lss_Chronolux_Help.new
  lss_chrono_help.add_help_cmd
  lss_chrono_help.add_visit_site_cmd
end

#-----------------------------------------------------------------------------
file_loaded("chrono_help.rb")