#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ chrono_init.rb ver. 1.0.  28-Sep-11
#~ The script, which performs initialization of global settings and puts them
#~ to the new model attributes or reads them from existing attributes if any

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# This class performs initializations of global Chronolux parameters

class Lss_Chronolux_Init
  attr_accessor :splitter_str
  
  attr_accessor :pnt_no 			# current test point number
  attr_accessor :short_ray_len		# the length of short rays
  attr_accessor :long_ray_len			# the length of long rays
  attr_accessor :ray_offset			# offset length: distance from test point
  attr_accessor :start_time
  attr_accessor :end_time
  attr_accessor :time_increment
  attr_accessor :calc_rays			# calculate or not insolation duration trigger
  attr_accessor :calc_svf			# calculate or not SVF trigger
  attr_accessor :calc_wvf			# not in use yet
  attr_accessor :svf_steps_cnt		# SVF sphere detail level
  attr_accessor :wvf_steps_cnt_x		# not in use yet
  attr_accessor :wvf_steps_cnt_y		# not in use yet
  attr_accessor :show_ray_calc		# show or hide calculation process trigger
  attr_accessor :expr_calc_factor		# the factor determines how less detailed express calculation than final calculation
  attr_accessor :ins_sector_type		# how to represent insolated sector (surface or rays cluster)
  attr_accessor :svf_sphere_r		# radius of SVF hemisphere
  attr_accessor :expr_calc_2dscl		# not in use yet
  attr_accessor :long_ray_len2d		# not in use yet
  attr_accessor :svf_sphere_r2d		# not in use yet
  
  attr_accessor :handle_transparency	# take in account transparency of faces material or not
  attr_accessor :transparency_threshold	# materials with greater alpha level than transparency_threshold are concidered transparent
  
  attr_accessor :chk_step_size		# chek points grid cell size (insolation of face test)
  
  attr_accessor :isoline_step			# how often isolines of tested face are to be placed
  attr_accessor :tst_f_results			# how to represent results of face insolation test
  
  # Initialize default values of global parameters
  def initialize
    @splitter_str=";" #to separate values inside strings (attributes and report generation use it value)
    
    @model=Sketchup.active_model
    
    @pnt_no=1
    @short_ray_len=1000.0
    @long_ray_len=2000.0
    @ray_offset=450.0
    
    @start_time=Time.at(1032678000.0) #Sun Sep 22 07:00:00 UTC 2002
    @end_time=Time.at(1032714000.0) #Sun Sep 22 17:00:00 UTC 2002
    @time_increment=60
    
    @calc_rays=true
    @calc_svf=false
    @calc_wvf=false
    
    @svf_steps_cnt=39
    @wvf_steps_cnt_x=30
    @wvf_steps_cnt_y=30
    
    @show_ray_calc=true
    
    @expr_calc_factor=5
    
    @ins_sector_type=$chronoluxStrings.GetString("Surface")
    @svf_sphere_r=500.0
    @expr_calc_2dscl=false
    
    @long_ray_len2d=500 #size in pixels to draw sun scale and express calc results at an independent scale
    @svf_sphere_r2d=500 #size in pixels to draw svf calc results at an independent scale
    
    @handle_transparency=false
    @transparency_threshold=0.5
    
    @chk_step_size=120.0
    
    @isoline_step=600 #step in seconds
    @tst_f_results=$chronoluxStrings.GetString("Isolines")
  end
  
  # Build "Global Settings" dialog and add "Global Settings" command into LSS Chronolux submenu and toolbar
  def add_global_settings_cmd
    settings_cmd=UI::Command.new($chronoluxStrings.GetString("Global Settings")){
      self.read_from_model #to ensure (maybe another file was opened and it has its own settings)
      prompts=Array.new
      prompts+= [$chronoluxStrings.GetString("Short ray length (current units) ")]
      prompts+= [$chronoluxStrings.GetString("Long ray length (current units) ")]
      prompts+= [$chronoluxStrings.GetString("Short ray offset (current units) ")]
      prompts+= [$chronoluxStrings.GetString("Calculate and draw rays")]
      prompts+= [$chronoluxStrings.GetString("Calculate SVF")]
      #~ prompts+= [$chronoluxStrings.GetString("Calculate WVF")]
      prompts+= [$chronoluxStrings.GetString("SVF steps count")]
      #~ prompts+= [$chronoluxStrings.GetString("WVF steps along X")]
      #~ prompts+= [$chronoluxStrings.GetString("WVF steps along Y")]
      prompts+= [$chronoluxStrings.GetString("Show rays checking process")]
      prompts+= [$chronoluxStrings.GetString("Express calculation factor")]
      prompts+= [$chronoluxStrings.GetString("Insolated sector representation")]
      prompts+= [$chronoluxStrings.GetString("SVF sphere radius")]
      #~ prompts+= [$chronoluxStrings.GetString("Show express calc. regardles to view scale ")]
      prompts+= [$chronoluxStrings.GetString("Handle transparency")]
      prompts+= [$chronoluxStrings.GetString("Transparency threshold")]
      prompts+= [$chronoluxStrings.GetString("Check step size")]
      prompts+= [$chronoluxStrings.GetString("Isolines step (in seconds)")]
      prompts+= [$chronoluxStrings.GetString("Test face results representation")]
      
      y_str=$chronoluxStrings.GetString("Yes")
      no_str=$chronoluxStrings.GetString("No")
      
      defaults = Array.new
      defaults+=[Sketchup.format_length(@short_ray_len)]
      defaults+=[Sketchup.format_length(@long_ray_len)]
      defaults+=[Sketchup.format_length(@ray_offset)]
      if @calc_rays
        defaults+=[y_str]
      else
        defaults+=[no_str]
      end
      if @calc_svf
        defaults+=[y_str]
      else
        defaults+=[no_str]
      end
      #~ if @calc_wvf
        #~ defaults+=[y_str]
      #~ else
        #~ defaults+=[no_str]
      #~ end
      defaults+=[@svf_steps_cnt]
      #~ defaults+ =[@wvf_steps_cnt_x]
      #~ defaults+=[@wvf_steps_cnt_y]
      if @show_ray_calc
        defaults+=[y_str]
      else
        defaults+=[no_str]
      end
      defaults+=[@expr_calc_factor]
      defaults+=[@ins_sector_type]
      defaults+=[Sketchup.format_length(@svf_sphere_r)]
      #~ if @expr_calc_2dscl
        #~ defaults+=[y_str]
      #~ else
        #~ defaults+=[no_str]
      #~ end
      if @handle_transparency
        defaults+=[y_str]
      else
        defaults+=[no_str]
      end
      defaults+=[@transparency_threshold]
      defaults+=[Sketchup.format_length(@chk_step_size)]
      defaults+=[@isoline_step]
      defaults+=[@tst_f_results]
      
      list=Array.new
      list+=[""] #@short_ray_len
      list+=[""] #@long_ray_len
      list+=[""] #@ray_offset
      list+=["#{y_str}|#{no_str}"] #@calc_rays
      list+=["#{y_str}|#{no_str}"] #@calc_svf
      #~ list+=["#{y_str}|#{no_str}"] #@calc_wvf
      list+=[""] #@svf_steps_cnt
      #~ list+=[""] #@wvf_steps_cnt_x
      #~ list+=[""] #@wvf_steps_cnt_y
      list+=["#{y_str}|#{no_str}"] #@show_ray_calc
      list+=[""] #@expr_calc_factor
      list+=["#{$chronoluxStrings.GetString("Surface")}|#{$chronoluxStrings.GetString("Rays")}"] #@ins_sector_type
      #~ list+=["#{y_str}|#{no_str}"] #@expr_calc_2dscl
      list+=[""] #@svf_sphere_r
      list+=["#{y_str}|#{no_str}"] #@handle_transparency
      list+=[""] #@transparency_threshold
      list+=[""] #@chk_step_size
      list+=[""] #@isoline_step
      list+=["#{$chronoluxStrings.GetString("Isolines")}|#{$chronoluxStrings.GetString("Gradient")}|#{$chronoluxStrings.GetString("Isolines and Gradient")}"] #@tst_f_results
      
	  # Patcth to solve encoding issue
	  @model.set_attribute "LSS Chronolux Settings", "y_str", y_str
	  y_str=@model.get_attribute "LSS Chronolux Settings", "y_str"
	  @model.set_attribute "LSS Chronolux Settings", "no_str", no_str
	  no_str=@model.get_attribute "LSS Chronolux Settings", "no_str"
	  # End of Patcth to solve encoding issue
	  
      input_res = UI.inputbox prompts, defaults, list, $chronoluxStrings.GetString("Global Settings")
      if input_res
        #it is necessary to add format checking to fields with numeric values!!!
        ind=0
        @short_ray_len=Sketchup.parse_length(input_res[ind]); ind+=1
        @long_ray_len=Sketchup.parse_length(input_res[ind]); ind+=1
        @ray_offset=Sketchup.parse_length(input_res[ind]); ind+=1
        if input_res[ind]==y_str
          @calc_rays=true
        else
          @calc_rays=false
        end; ind+=1
        if input_res[ind]==y_str
          @calc_svf=true
        else
          @calc_svf=false
        end; ind+=1
        #~ if input_res[ind]==y_str
          #~ @calc_wvf=true
        #~ else
          #~ @calc_wvf=false
        #~ end; ind+=1
        @svf_steps_cnt=input_res[ind]; ind+=1
        #~ @wvf_steps_cnt_x=input_res[ind]; ind+=1
        #~ @wvf_steps_cnt_y=input_res[ind]; ind+=1
        if input_res[ind]==y_str
          @show_ray_calc=true
        else
          @show_ray_calc=false
        end; ind+=1
        @expr_calc_factor=input_res[ind]; ind+=1
        @ins_sector_type=input_res[ind]; ind+=1
        @svf_sphere_r=Sketchup.parse_length(input_res[ind]); ind+=1
        #~ if input_res[ind]==y_str
          #~ @expr_calc_2dscl=true
        #~ else
          #~ @expr_calc_2dscl=false
        #~ end; ind+=1
        if input_res[ind]==y_str
          @handle_transparency=true
        else
          @handle_transparency=false
        end; ind+=1
        @transparency_threshold=input_res[ind]; ind+=1
        @chk_step_size=Sketchup.parse_length(input_res[ind]); ind+=1
	@isoline_step=input_res[ind]; ind+=1
	@tst_f_results=input_res[ind]; ind+=1
        self.write_to_model
      end
    }
    settings_cmd.small_icon = "./tb_icons/settings_16.png"
    settings_cmd.large_icon = "./tb_icons/settings_24.png"
    settings_cmd.tooltip = $chronoluxStrings.GetString("Click to set global settings")
    $chronoluxToolbar.add_separator
    $chronoluxToolbar.add_item(settings_cmd)
    $chronoluxMenu.add_separator
    $chronoluxMenu.add_item(settings_cmd)
  end
  
  #  Save global parameters as the active model attributes
  def write_to_model
    value=@model.set_attribute "LSS Chronolux Settings", "pnt_no", @pnt_no
    value=@model.set_attribute "LSS Chronolux Settings", "short_ray_len", @short_ray_len
    value=@model.set_attribute "LSS Chronolux Settings", "long_ray_len", @long_ray_len
    value=@model.set_attribute "LSS Chronolux Settings", "ray_offset", @ray_offset
    
    value=@model.set_attribute "LSS Chronolux Settings", "start_time", @start_time
    value=@model.set_attribute "LSS Chronolux Settings", "end_time", @end_time
    value=@model.set_attribute "LSS Chronolux Settings", "time_increment", @time_increment
    
    value=@model.set_attribute "LSS Chronolux Settings", "calc_rays", @calc_rays
    value=@model.set_attribute "LSS Chronolux Settings", "calc_svf", @calc_svf
    value=@model.set_attribute "LSS Chronolux Settings", "calc_wvf", @calc_wvf
    
    value=@model.set_attribute "LSS Chronolux Settings", "svf_steps_cnt", @svf_steps_cnt
    value=@model.set_attribute "LSS Chronolux Settings", "wvf_steps_cnt_x", @wvf_steps_cnt_x
    value=@model.set_attribute "LSS Chronolux Settings", "wvf_steps_cnt_y", @wvf_steps_cnt_y
    
    value=@model.set_attribute "LSS Chronolux Settings", "expr_calc_factor", @expr_calc_factor
    
    value=@model.set_attribute "LSS Chronolux Settings", "show_ray_calc", @show_ray_calc
    
    value=@model.set_attribute "LSS Chronolux Settings", "ins_sector_type", @ins_sector_type
    value=@model.set_attribute "LSS Chronolux Settings", "svf_sphere_r", @svf_sphere_r
    value=@model.set_attribute "LSS Chronolux Settings", "expr_calc_2dscl", @expr_calc_2dscl
    
    value=@model.set_attribute "LSS Chronolux Settings", "handle_transparency", @handle_transparency
    value=@model.set_attribute "LSS Chronolux Settings", "transparency_threshold", @transparency_threshold
    value=@model.set_attribute "LSS Chronolux Settings", "chk_step_size", @chk_step_size
    
    value=@model.set_attribute "LSS Chronolux Settings", "isoline_step", @isoline_step
    value=@model.set_attribute "LSS Chronolux Settings", "tst_f_results", @tst_f_results
  end
  
  # Read global parameters from model attributes if any
  def read_from_model
    #default values are added only for the 1-st time reading, if the initial model does not have such values, they'll be set to default values
    @pnt_no=@model.get_attribute "LSS Chronolux Settings", "pnt_no", 1
    @short_ray_len=@model.get_attribute "LSS Chronolux Settings", "short_ray_len", 1000.0
    @long_ray_len=@model.get_attribute "LSS Chronolux Settings", "long_ray_len", 2000.0
    @ray_offset=@model.get_attribute "LSS Chronolux Settings", "ray_offset", 450.0
    
    @start_time=@model.get_attribute "LSS Chronolux Settings", "start_time", Time.at(1032678000.0) #Sun Sep 22 07:00:00 UTC 2002
    @end_time=@model.get_attribute "LSS Chronolux Settings", "end_time", Time.at(1032714000.0) #Sun Sep 22 17:00:00 UTC 2002
    @time_increment=@model.get_attribute "LSS Chronolux Settings", "time_increment", 60
    
    @calc_rays=@model.get_attribute "LSS Chronolux Settings", "calc_rays", true
    @calc_svf=@model.get_attribute "LSS Chronolux Settings", "calc_svf", false
    @calc_wvf=@model.get_attribute "LSS Chronolux Settings", "calc_wvf", false
    
    @svf_steps_cnt=@model.get_attribute "LSS Chronolux Settings", "svf_steps_cnt", 39
    @wvf_steps_cnt_x=@model.get_attribute "LSS Chronolux Settings", "wvf_steps_cnt_x", 30
    @wvf_steps_cnt_y=@model.get_attribute "LSS Chronolux Settings", "wvf_steps_cnt_y", 30
    
    @expr_calc_factor=@model.get_attribute "LSS Chronolux Settings", "expr_calc_factor", 5
    
    @show_ray_calc=@model.get_attribute "LSS Chronolux Settings", "show_ray_calc", true
    
    @ins_sector_type=@model.get_attribute "LSS Chronolux Settings", "ins_sector_type", $chronoluxStrings.GetString("Surface")
    @svf_sphere_r=@model.get_attribute "LSS Chronolux Settings", "svf_sphere_r", 500.0
    @expr_calc_2dscl=@model.get_attribute "LSS Chronolux Settings", "expr_calc_2dscl", false
    
    @handle_transparency=@model.get_attribute "LSS Chronolux Settings", "handle_transparency", false
    @transparency_threshold=@model.get_attribute "LSS Chronolux Settings", "transparency_threshold", 0.5
    @chk_step_size=@model.get_attribute "LSS Chronolux Settings", "chk_step_size", 120.0
    
    @isoline_step=@model.get_attribute "LSS Chronolux Settings", "isoline_step", 600
    @tst_f_results=@model.get_attribute "LSS Chronolux Settings", "tst_f_results", $chronoluxStrings.GetString("Isolines")
  end
  
end #class Lss_Chronolux_Init

if( not file_loaded?("chrono_init.rb") )
  lss_chronolux=Lss_Chronolux_Init.new
  lss_chronolux.read_from_model #maybe the model already has some settings, so it is necessary to try to read them
  lss_chronolux.write_to_model
  lss_chronolux.add_global_settings_cmd
end

#-----------------------------------------------------------------------------
file_loaded("chrono_init.rb")