#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ chrono_time.rb ver. 1.0.  28-Sep-11
#~ The script, which adds time settings command buttons to the
#~ "LSS Chronolux Toolbar" and appropriate menu commands to "Chronolux" submenu.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

require 'chronolux/sun_scale.rb'

# This class builds dialogs to set insolation testing start time, end time and time increment (step of insolation testing)

class Lss_Chrono_Time_Cmds
	
  # Build time dialog and add time commands to LSS Chronolux submenu and toolbar
  def initialize
    @model=Sketchup.active_model
    $chronoluxToolbar.add_separator
    $chronoluxMenu.add_separator
    
    time_setup_cmd=UI::Command.new($chronoluxStrings.GetString("Time Settings")){
      @start_time=@model.get_attribute "LSS Chronolux Settings", "start_time", Time.at(1032678000.0) #Sun Sep 22 07:00:00 UTC 2002
      @end_time=@model.get_attribute "LSS Chronolux Settings", "end_time", Time.at(1032714000.0) #Sun Sep 22 17:00:00 UTC 2002
      @time_increment=@model.get_attribute "LSS Chronolux Settings", "time_increment", 60
      
      prompts=Array.new(3)
      prompts[0]=$chronoluxStrings.GetString("Start time")
      prompts[1]=$chronoluxStrings.GetString("End time")
      prompts[2]=$chronoluxStrings.GetString("Time increment, seconds")
      
      defaults = Array.new(3)
      defaults[0]=@start_time.utc.ctime
      defaults[1]=@end_time.utc.ctime
      defaults[2]=@time_increment
      
      input_res = UI.inputbox prompts, defaults, $chronoluxStrings.GetString("Time Settings")
      
      if input_res
        #requires Time class extention (located 'chronolux/3dparty') to enable 'parse' method
        @start_time=Time.parse(input_res[0]).utc+Time.parse(input_res[0]).gmtoff
        @end_time=Time.parse(input_res[1]).utc+Time.parse(input_res[1]).gmtoff
        @time_increment=input_res[2].to_i
        value=@model.set_attribute "LSS Chronolux Settings", "start_time", @start_time
        value=@model.set_attribute "LSS Chronolux Settings", "end_time", @end_time
        value=@model.set_attribute "LSS Chronolux Settings", "time_increment", @time_increment
        self.ask2regenerate_sun_scale
      end
    }
    time_setup_cmd.small_icon = "./tb_icons/time_setup_16.png"
    time_setup_cmd.large_icon = "./tb_icons/time_setup_24.png"
    time_setup_cmd.tooltip = $chronoluxStrings.GetString("Click to set start time, end time and time check step")
    $chronoluxToolbar.add_item(time_setup_cmd)
    $chronoluxMenu.add_item(time_setup_cmd)
    
    get_time_start_cmd=UI::Command.new($chronoluxStrings.GetString("Get Time Start")){
      model=Sketchup.active_model
      shadowinfo = model.shadow_info
      value = model.set_attribute "LSS Chronolux Settings", "start_time", shadowinfo["ShadowTime"]
      info_str=$chronoluxStrings.GetString("Start time has been set to: ")
      info_str+=shadowinfo["ShadowTime"].utc.to_s
      puts info_str
      self.ask2regenerate_sun_scale
    }
    get_time_start_cmd.small_icon = "./tb_icons/get_time_start_16.png"
    get_time_start_cmd.large_icon = "./tb_icons/get_time_start_24.png"
    get_time_start_cmd.tooltip = $chronoluxStrings.GetString("Click to get time start from current shadow settings")
    $chronoluxToolbar.add_item(get_time_start_cmd)
    $chronoluxMenu.add_item(get_time_start_cmd)
    
    get_time_end_cmd=UI::Command.new($chronoluxStrings.GetString("Get Time End")){
      model=Sketchup.active_model
      shadowinfo = model.shadow_info
      value = model.set_attribute "LSS Chronolux Settings", "end_time", shadowinfo["ShadowTime"]
      info_str=$chronoluxStrings.GetString("End time has been set to: ")
      info_str+=shadowinfo["ShadowTime"].utc.to_s
      puts info_str
      self.ask2regenerate_sun_scale
    }
    get_time_end_cmd.small_icon = "./tb_icons/get_time_end_16.png"
    get_time_end_cmd.large_icon = "./tb_icons/get_time_end_24.png"
    get_time_end_cmd.tooltip = $chronoluxStrings.GetString("Click to get time end from current shadow settings")
    $chronoluxToolbar.add_item(get_time_end_cmd)
    $chronoluxMenu.add_item(get_time_end_cmd)
  end
  
  # Ask to regenerate sun scale
  def ask2regenerate_sun_scale
    @start_time=@model.get_attribute "LSS Chronolux Settings", "start_time", Time.at(1032678000.0) #Sun Sep 22 07:00:00 UTC 2002
    @end_time=@model.get_attribute "LSS Chronolux Settings", "end_time", Time.at(1032714000.0) #Sun Sep 22 17:00:00 UTC 2002
    @time_increment=@model.get_attribute "LSS Chronolux Settings", "time_increment", 60
    steps_cnt=(@end_time-@start_time)/@time_increment
    message_str="#{$chronoluxStrings.GetString("Start time is set to:")} \t\t #{@start_time.utc} \n"
    message_str+="#{$chronoluxStrings.GetString("End time is set to:")} \t\t #{@end_time.utc} \n"
    message_str+="#{$chronoluxStrings.GetString("Time increment is set to:")} \t #{@time_increment} \n"
    message_str+="#{$chronoluxStrings.GetString("Total check rays at each test point count:")} #{steps_cnt.to_i} \n"
    message_str+="\n #{$chronoluxStrings.GetString("Is it necessary to regenerate sun scale?")}"
    message_str+="\n #{$chronoluxStrings.GetString("WARNING: start time is greater than end time!")}" if steps_cnt<0
    result = UI.messagebox(message_str, MB_YESNO)
    if result == 6 # Yes
      lss_sun_scale=Lss_Sun_Scale.new
      lss_sun_scale.generate_sun_scale
      lss_sun_scale.write_sun_scale2model
    end
  end
  
end #class Lss_Chrono_Time_Cmds

if( not file_loaded?("chrono_time.rb") )
  Lss_Chrono_Time_Cmds.new
end

#-----------------------------------------------------------------------------
file_loaded("chrono_time.rb")