#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ lss_utils.rb ver. 1.0.  08-Oct-11
#~ Utilities script, which contains some geometric calculation methods, progress bar class and Lss_Raytest class, which extends Sketchup raytest method.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# This is a class, which contains some geometric calculation methods

class Lss_Geom

  # Calculate the area of triangle by 3 given vertices
  def calc_triangle_area(pt1,pt2,pt3)
    a=pt1.distance(pt2)
    b=pt2.distance(pt3)
    c=pt3.distance(pt1)
    p=(a+b+c)/2.0
    formula=p*(p-a)*(p-b)*(p-c)
    if formula>0
      Math.sqrt(p*(p-a)*(p-b)*(p-c))
    else
      0
    end
  end
  
  # Transfer from polar coordinates to cartesian
  def cartesian(r, a_ang, b_ang)
    [r*Math.cos(a_ang)*Math.sin(b_ang), r*Math.sin(a_ang)*Math.sin(b_ang), r*Math.cos(b_ang)]
  end
end #class Lss_Geom

# This is a class, which contains small progress bar string generation

class Lss_Progr_Bar # Was renamed 08-Oct-11 to avoid name conflicts with lss_tools class Lss_Tiny_Progr_Bar
  attr_accessor :percents_ready
  attr_accessor :progr_string
  
  # Read input parameters
  def initialize(tot_cnt,progr_char,rest_char,scale_coeff)
    @scale_coeff=scale_coeff
    @scale_coeff=2 if @scale_coeff==nil or @scale_coeff==0
    if tot_cnt
      @tot_cnt=tot_cnt
      @tot_cnt=1 if @tot_cnt==0
    end
    @progr_char=progr_char
    @rest_char=rest_char
    @percents_ready=0
  end
  
  # Generate progress bar string using given input parameters
  def update(curr_cnt)
    @curr_cnt=curr_cnt
    @percents_ready=(100*@curr_cnt/(@tot_cnt)).round
    if 100/@scale_coeff-(@percents_ready/@scale_coeff)>= 0
      progr_str=@progr_char*((@percents_ready/@scale_coeff).round)+@rest_char*(100/@scale_coeff-(@percents_ready/@scale_coeff).round)
    else
      progr_str=@progr_char*((100/@scale_coeff).round)
    end
    @progr_string="#{@percents_ready}% #{progr_str}"
  end
end #class Lss_Tiny_Progr_Bar

# This is a class, which extends "Sketchup.active_model.raytest" method. "Lss_Raytest.test" discards transparent objects
# (calculation results and faces with transparent material)

class Lss_Raytest
  
  # Get ray and read transparency settings from active model attributes
  def initialize(ray)
    @ray=ray
    @model=Sketchup.active_model
    @handle_transparency=@model.get_attribute "LSS Chronolux Settings", "handle_transparency", false
    @transparency_threshold=@model.get_attribute "LSS Chronolux Settings", "transparency_threshold", 0.5
  end
  
  # Perform test
  def test
    transp=true
    while transp
      intersect_res=@model.raytest(@ray)
      if intersect_res
        inst_path=intersect_res[1]
        inst_path.each{|inst| 
          transp=inst.get_attribute "LSS Chronolux", "transparent"
          if @handle_transparency
            mat=inst.material
            if mat
              alpha=mat.alpha 
              transp=true if alpha<=@transparency_threshold
            end
          end
          break if transp
        }
        if transp
	  @ray[0]=Geom::Point3d.new(intersect_res[0])
        end
      else
        break
      end
    end
    result=@model.raytest(@ray)
    result
  end
  
end #class Lss_Raytest