#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ report_maker.rb ver. 1.0.  28-Sep-11
#~ The script, which contains report generation class

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# This class collects test results information from a given test point group attributes and write results to 
# instance variables. It is more easy to process a set of such Lss_Report_Test_Pnt class instances
# while building a report string, than to process a set of groups in Sketchup.active_model

class Lss_Report_Test_Pnt
  attr_accessor :pnt_no
  attr_accessor :total_ins_duration
  attr_accessor :svf
  attr_accessor :wvf
  attr_accessor :ins_intervals_array
  
  # Read attributes from test point group attributes and write them to instance variables
  def initialize(test_pnt_group)
    return if test_pnt_group.deleted? or test_pnt_group.nil? #added after some error messages 14.09.10
    @dictionary=test_pnt_group.attribute_dictionaries["LSS Test Point Properties"]
    @pnt_no=test_pnt_group.get_attribute "LSS Test Point Properties", "pnt_no"
    @total_ins_duration=test_pnt_group.get_attribute "LSS Test Point Properties", "total_ins_duration"
    @svf=test_pnt_group.get_attribute "LSS Test Point Properties", "svf"
    @wvf=test_pnt_group.get_attribute "LSS Test Point Properties", "wvf"
    @ins_intervals_array=Array.new
    for key in @dictionary.keys
      # Do not show attributes that start with _, as these are internal.
      if key[0..0] != '_'
        splt_key=key.split(" ")
        if splt_key.length>1
          if splt_key[0]=="ins_interval"
            @ins_intervals_array+=[@dictionary[key]]
          end
        end
      end
    end
  end
end #class Lss_Report_Test_Pnt

# This is a class, which contains report generator (it lists all or selected test points results in active model)

class Lss_Chrono_Report
  attr_accessor :report_str
  attr_accessor :title_str
  
  def initialize(test_pnt_grps_array)
    @title_array=Array.new
    @title_array+=[$chronoluxStrings.GetString("Pt#")]
    @title_array+=[$chronoluxStrings.GetString("Int#")]
    @title_array+=[$chronoluxStrings.GetString("  Starts ")]
    @title_array+=[$chronoluxStrings.GetString("   Ends   ")]
    @title_array+=[$chronoluxStrings.GetString(" Duration  ")]
    @title_array+=[$chronoluxStrings.GetString("SVF, %")]
    @title_array+=[$chronoluxStrings.GetString("WVF, %")]
    @report_str=""
    @global_settings=Lss_Chronolux_Init.new
    @splitter_str=@global_settings.splitter_str
    @test_pnt_grps_array=Array.new(test_pnt_grps_array)
    @test_pnts_list=Array.new
    @report_lines=Array.new
    #flags to find out if attributes exists to generate or not appropriate columns
    @has_total_ins_duration=false
    @has_intervals_array=false
    @has_svf=false
    @has_wvf=false
  end
  
  # Build report string.
  def build_report_str
    # Make a set of list-friendly Lss_Report_Test_Pnt instances from a set of test point groups in active model
    @test_pnt_grps_array.each{|test_pnt_group|
      report_test_pnt=Lss_Report_Test_Pnt.new(test_pnt_group)
      @test_pnts_list+=[report_test_pnt]
    }
    # Fill @report_lines array with data collected from @test_pnts_list
    @test_pnts_list.sort!{|a,b| a.pnt_no.to_i <=> b.pnt_no.to_i}
    @test_pnts_list.each{|test_pnt|
      report_line=Array.new
      if test_pnt.pnt_no.to_s.length==2
        report_line+=[test_pnt.pnt_no.to_s]
      else
        report_line+=[" " + test_pnt.pnt_no.to_s]
      end
      report_line+=[test_pnt.ins_intervals_array]
      if test_pnt.total_ins_duration
        total_ins_duration=Time.at(test_pnt.total_ins_duration).utc
        h=$chronoluxStrings.GetString("h")
        m=$chronoluxStrings.GetString("m")
        duration_str="#{total_ins_duration.hour.to_s}#{h}#{total_ins_duration.min.to_s}#{m}"
      else
        duration_str=""
      end
      report_line+=[duration_str]
      report_line+=[test_pnt.svf] if test_pnt.svf
      report_line+=[test_pnt.wvf] if test_pnt.wvf
      @report_lines+=[report_line]
    }
    
    # Build the report string
    columns_cnt=0
    @report_lines.each{|line|
      columns_cnt=line.length if line.length>columns_cnt
      if line[1].length==0
        @report_str+="#{line[0]}  \t __ \t  _____  \t  _____ \t  ______   \t"
        @report_str+=" #{line[3]} " if line[3]
        @report_str+="\t #{line[4]} " if line[4]
        @report_str+="\n"
      else
        line[1].each{|interval|
          int_line=interval.split(";")
          int_str=""
          int_line.each{|cell| int_str+="  #{cell} \t "}
          if int_line[0].to_i>1
            @report_str+=" __ \t #{int_str}"
          else
            @report_str+="#{line[0]}  \t #{int_str}"
          end
          @report_str+="#{line[3]} " if line[3]
          @report_str+="\t #{line[4]} " if line[4]
          @report_str+="\n"
        }
      end
    }
    @title_str=""
    column_no=0
    while column_no<columns_cnt+2
      @title_str+=@title_array[column_no]+"\t"
      column_no+=1
    end
    @title_str.chop!
    @title_str+="\n"
	# Patcth to solve encoding issue
	  @model=Sketchup.active_model
	  @model.set_attribute "LSS Chronolux Settings", "title_str", @title_str
	  @title_str=@model.get_attribute "LSS Chronolux Settings", "title_str"
	  # End of Patcth to solve encoding issue
    @report_str=@title_str+@report_str
  end
  
  # Put report copy to Ruby console
  def report2console
    puts @report_str
  end
end #class Lss_Chrono_Report