#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ sun_scale.rb ver. 1.0.  08-Oct-11
#~ The script, which generates the sun scale and writes it to the active model or reads the existing sun scale from the model

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# This is a class, which generates the sun scale and writes it to the active model or reads the existing sun scale from the model

class Lss_Sun_Scale
  attr_accessor :sun_scale_times
  attr_accessor :sun_scale_dir_vecs
  
  # Read global settings from active model attributes and initialize arrays of test times 
  # and corresponding vectors from test point to sun locations
  def initialize
    @model=Sketchup.active_model
    @global_settings=Lss_Chronolux_Init.new
    @global_settings.read_from_model
    @splitter_str=@global_settings.splitter_str
    
    @shadowinfo = @model.shadow_info
    
    @short_ray_len=@global_settings.short_ray_len
    @long_ray_len=@global_settings.long_ray_len
    @ray_offset=@global_settings.ray_offset
    @start_time=@global_settings.start_time
    @end_time=@global_settings.end_time
    @time_increment=@global_settings.time_increment
    
    @sun_scale_times=Array.new
    @sun_scale_dir_vecs=Array.new
  end
  
  # Iterate through test time period (using @time_increment) and get an apropriate sun direction from active_model.shadow_info
  # for each testing time, then fill arrays of testing times and corresponding sun direction vectors
  def generate_sun_scale
    chk_steps_cnt=(@end_time-@start_time)/@time_increment.to_f
    prgr_bar=Lss_Progr_Bar.new(chk_steps_cnt,"|","_",2)
    curr_step=0
    @curr_time=@start_time
    while @curr_time<=@end_time
      @shadowinfo["ShadowTime"]=@curr_time
      sun_dir=@shadowinfo["SunDirection"]
      @sun_scale_times+=[@curr_time.utc]
      @sun_scale_dir_vecs+=[sun_dir]
      @curr_time+=@time_increment
      curr_step+=1
      prgr_bar.update(curr_step)
      result = Sketchup.status_text = "#{$chronoluxStrings.GetString("Generating sun scale:")} #{prgr_bar.progr_string}"
    end
    if @curr_time<@end_time
      @shadowinfo["ShadowTime"]=@end_time
      sun_dir=@shadowinfo["SunDirection"]
      @sun_scale_times+=[@end_time.utc]
      @sun_scale_dir_vecs+=[sun_dir]
    end
    Sketchup.status_text = ""
  end
  
  # Save generated arrays of testing times and sun direction vectors to an active_model attributes
  def write_sun_scale2model
    sun_times_dic=@model.attribute_dictionary("LSS Chronolux Sun Scale Times",true)
    sun_dir_vecs_dic=@model.attribute_dictionary("LSS Chronolux Sun Scale Dir Vecs",true)
    #-------clearing from previous sun scale information if any
    clearing_str=$chronoluxStrings.GetString("Clearing from previous sun scale...")
    dots_str=""
    if sun_times_dic.keys.length>0
      sun_times_dic.each_key{|key| 
        sun_times_dic.delete_key(key)
        dots_str+="."
        dots_str="" if dots_str.length>=100
        Sketchup.status_text = clearing_str+dots_str
      } 
    end
    if sun_dir_vecs_dic.keys.length>0
      sun_dir_vecs_dic.each_key{|key| 
        sun_dir_vecs_dic.delete_key(key)
        dots_str+="."
        dots_str="" if dots_str.length>=100
        Sketchup.status_text = clearing_str+dots_str
      }
    end
    #-------end clearing from previous sun scale information if any
    prgr_bar=Lss_Progr_Bar.new(@sun_scale_times.length,"|","_",2)
    @sun_scale_times.to_a.each_index{|ray_ind|
      sun_ray_ind_str="sun ray no "+ray_ind.to_s
      @sun_time_str=@sun_scale_times[ray_ind].to_f.to_s
      @sun_dir_vec_str=""
      @sun_scale_dir_vecs[ray_ind].to_a.each{|crd| @sun_dir_vec_str+=crd.to_s+@splitter_str}
      value=@model.set_attribute "LSS Chronolux Sun Scale Times", sun_ray_ind_str, @sun_time_str
      value=@model.set_attribute "LSS Chronolux Sun Scale Dir Vecs", sun_ray_ind_str, @sun_dir_vec_str
      prgr_bar.update(ray_ind)
      result = Sketchup.status_text = "#{$chronoluxStrings.GetString("Writing sun scale to the model:")} #{prgr_bar.progr_string}"
    }
    Sketchup.status_text = ""
  end
  
  # Read testing times and corresponding sun direction vectors from active model attributes to arrays
  # (the point is that array is more friendly data representation in this case, it's more easy to process hereinafter)
  def read_sun_scale_from_model
    reading_str = $chronoluxStrings.GetString("Reading sun scale...")
    dots_str=""
    @sun_scale_times=Array.new
    @sun_scale_dir_vecs=Array.new
    sun_times_dic=@model.attribute_dictionary("LSS Chronolux Sun Scale Times")
    sun_dir_vecs_dic=@model.attribute_dictionary("LSS Chronolux Sun Scale Dir Vecs")
    times_arr=Array.new
    if sun_times_dic
      sun_times_dic.each_key{|key|
        pair=Array.new
        pair+=[key]; pair+=[sun_times_dic[key]]
        times_arr+=[pair]
        dots_str+="."
        dots_str="" if dots_str.length>=100
        Sketchup.status_text = reading_str+dots_str
      }
    end
    times_arr.sort!{|a,b| a[0].to_s.split.last.to_i <=> b[0].to_s.split.last.to_i}
    times_arr.each{|pair| @sun_scale_times+=[Time.at(pair[1].to_f).utc]}
    
    vecs_arr=Array.new
    if sun_dir_vecs_dic
      sun_dir_vecs_dic.each_key{|key|
        pair=Array.new
        vec_arr=sun_dir_vecs_dic[key].split(@splitter_str)
        vec=Geom::Vector3d.new(vec_arr.x.to_f, vec_arr.y.to_f, vec_arr.z.to_f)
        pair+=[key]; pair+=[vec]
        vecs_arr+=[pair]
        dots_str+="."
        dots_str="" if dots_str.length>=100
        Sketchup.status_text = reading_str+dots_str
      }
    end
    vecs_arr.sort!{|a,b| a[0].to_s.split.last.to_i <=> b[0].to_s.split.last.to_i}
    vecs_arr.each{|pair| @sun_scale_dir_vecs+=[pair[1]]}
    Sketchup.status_text = ""
  end
  
end #class Lss_Sun_Scale