#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ svf_calc.rb ver. 1.0.  28-Sep-11
#~ SVF calculation script. It contains methods to trace rays from a given point and to calculate SVF.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# This is a class, which contains SVF calculation

class Lss_SVF
  attr_accessor :svf_sphere_r
  attr_accessor :svf_steps_cnt
  attr_accessor :rays_array
  attr_accessor :results_array
  attr_accessor :sphere_pnts
  attr_accessor :segm_corners
  attr_accessor :hemisphere_area
  
  def initialize(pnt_pos, svf_sphere_r, svf_steps_cnt)
    @model=Sketchup.active_model
    @position=Geom::Point3d.new(pnt_pos)
    @svf_sphere_r=svf_sphere_r
    @svf_steps_cnt=svf_steps_cnt
    
    @delta_ang=2.0*Math::PI/(@svf_steps_cnt.to_f)
    
    @rays_array=Array.new
    @sphere_pnts=Array.new
    @segm_corners=Array.new
  end
  
  # Build rays array. Each ray begins at a given test point and rays evenly fill the hemisphere above the given test point.
  # Rays density depends on @svf_steps_cnt and as a result on @delta_ang.
  def build_rays_array
    b_ang=Math::PI/2.0
    while b_ang>0
      a_ang=0.0
      while a_ang<2.0*Math::PI-@delta_ang/4.0 #sometimes steps count becomes different from @svf_steps_cnt
        x=Lss_Geom.new.cartesian(@svf_sphere_r,a_ang+@delta_ang/2.0,b_ang-@delta_ang/2.0)[0]
        y=Lss_Geom.new.cartesian(@svf_sphere_r,a_ang+@delta_ang/2.0,b_ang-@delta_ang/2.0)[1]
        z=Lss_Geom.new.cartesian(@svf_sphere_r,a_ang+@delta_ang/2.0,b_ang-@delta_ang/2.0)[2]
        ray_vec=Geom::Vector3d.new(x,y,z)
        @rays_array+=[ray_vec]
        @sphere_pnts+=[@position.offset(ray_vec)]
        
        x=Lss_Geom.new.cartesian(@svf_sphere_r,a_ang,b_ang)[0]
        y=Lss_Geom.new.cartesian(@svf_sphere_r,a_ang,b_ang)[1]
        z=Lss_Geom.new.cartesian(@svf_sphere_r,a_ang,b_ang)[2]
        ray_vec=Geom::Vector3d.new(x,y,z)
        @segm_corners+=[@position.offset(ray_vec)]
        a_ang+=@delta_ang
      end
      b_ang+=-@delta_ang
    end
  end
  
  # Perform ray test and fill @results_array with test results
  def perform_ray_test
    @results_array=Array.new(@rays_array.length)
    @rays_array.each_index{|ray_ind|
      ray_vec=@rays_array[ray_ind]
      chk_ray=[@position, ray_vec]
      intersect_res=Lss_Raytest.new(chk_ray).test
      @results_array[ray_ind]=intersect_res[0] if intersect_res
    }
  end
  
  # Calculate hemisphere area. It is the area of approximated hemisphere and it can not be calculated only by its radius,
  # because it depends on flat segments count too. The program uses obtained hemisphere area to calculate SVF subsequently.
  def calc_hemisphere_area
    @hemisphere_area=0.0
    @sphere_pnts.each_index{|ray_ind|
      sph_pnt=Geom::Point3d.new(@sphere_pnts[ray_ind])
      if ray_ind<@sphere_pnts.length-@svf_steps_cnt
        chk_mod=(ray_ind+@svf_steps_cnt+1).divmod(@svf_steps_cnt)[1]
        if chk_mod!=0
          segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
          segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind+1])
          segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
          segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt+1])
        else
          segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
          segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind-@svf_steps_cnt+1])
          segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
          segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+1])
        end
        area1=Lss_Geom.new.calc_triangle_area(segm_pnt1,segm_pnt2,segm_pnt3)
        area2=Lss_Geom.new.calc_triangle_area(segm_pnt2,segm_pnt4,segm_pnt3)
        @hemisphere_area+=area1+area2
      end
    }
  end
  
end #class Lss_SVF