#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ test_face.rb ver. 1.0.  08-Oct-11
#~ Test face script. It contains Test Face Tool and methods to process batch testing of the given area 
#~ defined by picked face

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

require 'chronolux/sun_scale.rb'

require 'chronolux/layers.rb'

require 'chronolux/test_point.rb'

# This is a class, which adds test face command into the LSS Chronolux submenu and into the toolbar

class Lss_Test_Face_Cmds
  def initialize
    @model=Sketchup.active_model
    $chronoluxToolbar.add_separator
    $chronoluxMenu.add_separator
    pick_test_face_cmd=UI::Command.new($chronoluxStrings.GetString("Pick Test Face...")){
      pick_test_face_tool=Lss_Pick_Test_Face_Tool.new
      Sketchup.active_model.select_tool pick_test_face_tool
      dir=pick_test_face_tool.getInstructorContentDirectory #does not work
    }
    pick_test_face_cmd.small_icon = "./tb_icons/pick_test_face_16.png"
    pick_test_face_cmd.large_icon = "./tb_icons/pick_test_face_24.png"
    pick_test_face_cmd.tooltip = $chronoluxStrings.GetString("Click to pick a test face")
    $chronoluxToolbar.add_item(pick_test_face_cmd)
    $chronoluxMenu.add_item(pick_test_face_cmd)
  end
end #class Lss_Test_Face_Cmds

# This is a class, which implements the tool for insolation duration testing of an area defined by a face in an active model

class Lss_Pick_Test_Face_Tool
  attr_accessor :lss_tool_name
  def initialize
    @process_anyway=false #flag, which enforces test net generation even if its too dense
    @pre_calc_pnts_cnt=0.0
    @test_pnts2draw_lim=2000 #maybe it is reasonable to make this value adjustable by user, maybe not
    @lss_tool_name="Chronolux Pick Test Face Tool" # program subsequently uses this parameter for help context sensivity
    @model=Sketchup.active_model
    @shadowinfo = @model.shadow_info
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    cur_path=Sketchup.find_support_file("pick_test_face.png", "Plugins/Chronolux/cursors/")
    @pick_test_face_cur_id=UI.create_cursor(cur_path, 0, 0)
  end
  
  def reset(view)
    @ip = nil
    @ip1 = nil
    @face2pick_pnts=nil
    @picked_face_pnts=nil
    @calc_in_progress=false
    @results_array=nil
    @test_net=nil
    @corners4result=nil
  end
  
  def onSetCursor
    UI.set_cursor(@pick_test_face_cur_id)
  end
  
  def activate
    @model.set_attribute("LSS Chronolux Settings", "lss_tool_name", @lss_tool_name) # program subsequently uses this parameter for help context sensivity
    @chrono_layers=Lss_Chrono_Layers.new
    @global_settings=Lss_Chronolux_Init.new
    @global_settings.read_from_model
    @chk_step_size=@global_settings.chk_step_size
    @time_increment=@global_settings.time_increment
    @show_ray_calc=@global_settings.show_ray_calc
    @long_ray_len=@global_settings.long_ray_len
    #~ @start_time=@global_settings.start_time
    #~ @end_time=@global_settings.end_time
    @isoline_step=@global_settings.isoline_step
    @tst_f_results=@global_settings.tst_f_results
    @test_net=Array.new # array of test points
    @under_cursor=nil
    @selected_face=nil
    @selected_face_plane=nil
    @selected_face_norm=nil
    @face2pick_pnts=Array.new
    @picked_face_pnts=Array.new
    @calc_in_progress=false
    @results_array=Array.new
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    self.obtain_sun_scale
  end
  
  def obtain_sun_scale
    @lss_sun_scale=Lss_Sun_Scale.new
    @lss_sun_scale.read_sun_scale_from_model
    @sun_scale_times=Array.new(@lss_sun_scale.sun_scale_times)
    @sun_scale_dir_vecs=Array.new(@lss_sun_scale.sun_scale_dir_vecs)
    if @sun_scale_times.length==0
      @lss_sun_scale.generate_sun_scale
      @lss_sun_scale.write_sun_scale2model
      @sun_scale_times=Array.new(@lss_sun_scale.sun_scale_times)
      @sun_scale_dir_vecs=Array.new(@lss_sun_scale.sun_scale_dir_vecs)
    end
    @start_time=@sun_scale_times.first
    @end_time=@sun_scale_times.last
  end
  
  # Generate a grid made of test points within picked face boundary (grid cell size = @check_step_size)
  # Generate accessorial array of corner points too (4 corners to define a square around each test point)
  # Programm subsequently uses array of corner points to represent calculation results
  def generate_test_net(process_anyway=false)
    @test_net=Array.new
    @corners4result=Array.new
    face=@under_cursor
    chk_area=@chk_step_size*@chk_step_size
    @pre_calc_pnts_cnt=face.area/chk_area # estimate the grid density before its generation
    return if @pre_calc_pnts_cnt>@test_pnts2draw_lim and not process_anyway # break operation if the grid is too dense unless generation is enforced
    edges=face.edges
    norm=face.normal
    align_tr=Geom::Transformation.new([0,0,0], norm).inverse # transformation of face aligning to horizontal plane
    aligned_face_pnts=Array.new
    @face2pick_pnts.each{|pt| aligned_face_pnts+=[pt.transform(align_tr)]}
    aligned_bnds=Geom::BoundingBox.new
    aligned_bnds.add(aligned_face_pnts)
    aligned_test_net=Array.new
    aligned_corners=Array.new
    z_pos=aligned_bnds.center.z
    x_pos=aligned_bnds.min.x
    
    # Generate the grid aligned to horizontal plane
    while x_pos<=aligned_bnds.max.x
      y_pos=aligned_bnds.min.y
      while y_pos<=aligned_bnds.max.y
        aligned_test_net+=[Geom::Point3d.new(x_pos,y_pos,z_pos)]
        crn1=Geom::Point3d.new(x_pos+@chk_step_size/2.0,y_pos+@chk_step_size/2.0,z_pos)
        crn2=Geom::Point3d.new(x_pos+@chk_step_size/2.0,y_pos-@chk_step_size/2.0,z_pos)
        crn3=Geom::Point3d.new(x_pos-@chk_step_size/2.0,y_pos-@chk_step_size/2.0,z_pos)
        crn4=Geom::Point3d.new(x_pos-@chk_step_size/2.0,y_pos+@chk_step_size/2.0,z_pos)
        square_crns=[crn1,crn2,crn3,crn4]
        aligned_corners+=[square_crns]
        y_pos+=@chk_step_size
      end
      x_pos+=@chk_step_size
    end
    
    # Align horizontal test grid to picked face
    aligned_test_net.each{|pt| pt.transform!(align_tr.inverse)}
    aligned_corners.each{|sq| sq.each{|pt| pt.transform!(align_tr.inverse)}}
    
    # Leave only those test points which get into face boundaries
    aligned_test_net.each_index{|pt_ind|
      pt=aligned_test_net[pt_ind]
      result = face.classify_point(pt)
      if result==1
        @test_net+=[pt]
        @corners4result+=[aligned_corners[pt_ind]]
      end
    }
  end
  
  def onMouseMove(flags, x, y, view)
    @ip1.pick view, x, y
    if( @ip1 != @ip )
      view.invalidate
      @ip.copy! @ip1
      view.tooltip = @ip.tooltip
    end
    
    if @selected_face.nil?
      @under_cursor=nil
      ph=view.pick_helper
      ph.do_pick x,y
      ph.all_picked.each{|ent|
        if ent.typename=="Face"
          @under_cursor=ent
          verts=ent.outer_loop.vertices
          @face2pick_pnts=Array.new
          verts.each{|vrt| @face2pick_pnts+=[vrt.position]}
          self.generate_test_net
          break
        else
          @face2pick_pnts=nil
        end
      }
      @under_cursor = ph.best_picked if @under_cursor.nil?
      if @under_cursor
        Sketchup.set_status_text("#{@under_cursor.typename} #{$chronoluxStrings.GetString("is under the cursor.")}")
        view.invalidate
      else
        Sketchup.set_status_text($chronoluxStrings.GetString("Click on face to start testing its surface."))
      end
    else
      @face2pick_pnts=nil
    end
  end
  
  def draw(view)
    if @ip.valid?
      @ip.draw(view)
    end
    
    # Highlight a face, wich is currently under the mouse pointer
    if @face2pick_pnts
      if @face2pick_pnts.length>2
        view.draw(GL_POLYGON, @face2pick_pnts) 
        view.draw_points @face2pick_pnts, 5, 2, "red"
      end
    end
    
    # Inform how many test points lay within selected test face
    if @test_net and @calc_in_progress==false
      txt_pt=view.screen_coords(@ip.position)+[20,20]
      view.draw_text(txt_pt, "")
      view.draw_text(txt_pt, "#{$chronoluxStrings.GetString("Test points count:")} #{@test_net.length}")
      col=[150,150,300]
      view.draw_points @test_net, 5, 3, col if @test_net.length>0
    end
    
    # Warn about test grid density excess
    if @pre_calc_pnts_cnt>@test_pnts2draw_lim
      txt_pt=view.screen_coords(@ip.position)+[20,20]
      view.draw_text(txt_pt, "")
      cnt_str=@pre_calc_pnts_cnt.to_i
      warn_str="#{$chronoluxStrings.GetString("Test points count:")} #{cnt_str}\n"
      warn_str+="#{$chronoluxStrings.GetString("Test grid is too dense to display")}\n"
      warn_str+="#{$chronoluxStrings.GetString("Warning: it could take a lot of time to calculate!")}"
      view.draw_text(txt_pt, warn_str)
    end
    if @picked_face_pnts
      view.drawing_color = "red"
      view.line_width = 3
      view.draw_polyline(@picked_face_pnts) if @picked_face_pnts.length>2
    end
    
    # Show calculation results if any during calculation while tool is active
    if @results_array
      if @results_array.length>0
        col_step=200.0/@max_duration
        @corners4result.each_index{|pt_ind|
          square_pts=Array.new(@corners4result[pt_ind])
          square_pts2d=Array.new
          square_pts.each{|pt| square_pts2d+=[view.screen_coords(pt)]}
          res=@results_array[pt_ind]
          res_col=50+(res*col_step).to_i
          col=[res_col,res_col,res_col]
          status = view.drawing_color=col
          view.draw2d(GL_POLYGON, square_pts2d)
        }
        if @sun_dir_vec
          sun_vec=Geom::Vector3d.new(@sun_dir_vec)
          pt1=Geom::Point3d.new(@selected_face.bounds.center)
          len=@long_ray_len
          sun_vec.length=len*2.0
          pt2=pt1.offset(sun_vec)
          
          # text does not appear if the command was placed once ------------
          txt_pt=view.screen_coords(pt2)
          status = view.drawing_color="black"
          view.draw_text(txt_pt, @time_str)
          view.draw_text(txt_pt, @time_str)
          #------------
          
          view.drawing_color="red"
          view.line_width = 1
          view.draw_line(pt1,pt2)
          view.draw_points pt2, 10, 5,"red"
        end
      end
    end
  end
  
  # Process picked test face (runs after clicking on the face)
  def process_test_face(view)
    @max_duration=@end_time-@start_time
    @results_array=Array.new(@test_net.length,@max_duration)
    if @show_ray_calc
      self.show_test_process(view)
    else
      self.hide_test_process(view)
    end
  end
  
  # Use timer to show how calculation processes
  def show_test_process(view)
    prgr_bar=Lss_Progr_Bar.new(@sun_scale_times.length,"|","_",2)
    @calc_in_progress=true
    ray_ind=0
    test_face_timer_id=UI.start_timer(0.01,true){
      @curr_time=@sun_scale_times[ray_ind]
      @shadowinfo["ShadowTime"]=@curr_time
      @sun_dir_vec=@sun_scale_dir_vecs[ray_ind]
      prgr_bar.update(ray_ind)
      hr=@curr_time.hour.to_s; hr="0"+hr if hr.length==1
      mn=@curr_time.min.to_s; mn="0"+mn if mn.length==1
      @time_str="#{hr}:#{mn}"
      @time_and_progr="#{@time_str} #{prgr_bar.progr_string}"
      Sketchup.status_text = @time_and_progr
      if @test_net
        @test_net.each_index{|pt_ind|
          pt=Geom::Point3d.new(@test_net[pt_ind])
          sun_ray=[pt,Geom::Vector3d.new(@sun_dir_vec)]
          intersect_res=Lss_Raytest.new(sun_ray).test
          @results_array[pt_ind]=@results_array[pt_ind]-@time_increment if intersect_res
        }
      end
      view.invalidate
      if ray_ind==@sun_scale_times.length-1 or @test_net.nil?
        self.generate_results_group if @test_net
        self.reset(view)
        view.invalidate
        Sketchup.active_model.select_tool nil
        @calc_in_progress=false
        test_face_timer_id=UI.stop_timer(test_face_timer_id)
      end
      ray_ind+=1
    }
  end
  
  # Use iteration through testing times (does not show calculation process in contrast to timer)
  def hide_test_process(view)
    prgr_bar=Lss_Progr_Bar.new(@sun_scale_times.length,"|","_",2)
    @calc_in_progress=true
    @sun_scale_times.each_index{|ray_ind|
      @curr_time=@sun_scale_times[ray_ind]
      @sun_dir_vec=@sun_scale_dir_vecs[ray_ind]
      prgr_bar.update(ray_ind)
      hr=@curr_time.hour.to_s; hr="0"+hr if hr.length==1
      mn=@curr_time.min.to_s; mn="0"+mn if mn.length==1
      @time_str="#{hr}:#{mn}"
      @time_and_progr="#{@time_str} #{prgr_bar.progr_string}"
      Sketchup.status_text = @time_and_progr
      @test_net.each_index{|pt_ind|
        pt=Geom::Point3d.new(@test_net[pt_ind])
        sun_ray=[pt,Geom::Vector3d.new(@sun_dir_vec)]
        intersect_res=Lss_Raytest.new(sun_ray).test
        @results_array[pt_ind]=@results_array[pt_ind]-@time_increment if intersect_res
      }
    }
    self.generate_results_group
    self.reset(view)
    view.invalidate
    Sketchup.active_model.select_tool nil
    @calc_in_progress=false
  end
  
  # Generate a group in an active model, which represents insolation calculation results
  def generate_results_group
    @model.start_operation($chronoluxStrings.GetString("Generate Test Face Results"))
    @test_face_grp=@model.entities.add_group
    @test_face_grp.name="LSS Test Face Group"
    @test_face_grp.layer=@chrono_layers.test_face_layer
	
	# Patcth to solve encoding issue
	grad_str=$chronoluxStrings.GetString("Gradient")
	@model.set_attribute "LSS Chronolux Settings", "grad_str", grad_str
	grad_str=@model.get_attribute "LSS Chronolux Settings", "grad_str"
	isoln_str=$chronoluxStrings.GetString("Isolines")
	@model.set_attribute "LSS Chronolux Settings", "isoln_str", isoln_str
	isoln_str=@model.get_attribute "LSS Chronolux Settings", "isoln_str"
	both_str=$chronoluxStrings.GetString("Isolines and Gradient")
	@model.set_attribute "LSS Chronolux Settings", "both_str", both_str
	both_str=@model.get_attribute "LSS Chronolux Settings", "both_str"
	# End of Patcth to solve encoding issue
    
    self.generate_gradient if @tst_f_results==grad_str or @tst_f_results==both_str
    self.generate_isolines if @tst_f_results==isoln_str or @tst_f_results==both_str
    
    @average_duration=@average_duration/@results_array.length.to_f
    @average_duration=Time.at(@average_duration).utc
    txt_pos=@test_face_grp.bounds.max
    h=$chronoluxStrings.GetString("h")
    m=$chronoluxStrings.GetString("m")
    hr=@average_duration.hour.to_s; hr="0"+hr if hr.length==1
    mn=@average_duration.min.to_s; mn="0"+mn if mn.length==1
    duration_str="#{$chronoluxStrings.GetString("Average insolation duration:")} #{hr}#{h}#{mn}#{m}"
    duration_text=@test_face_grp.entities.add_text(duration_str, txt_pos)
    duration_text.layer=@chrono_layers.test_face_layer
    
    Sketchup.status_text = $chronoluxStrings.GetString("Test face result is ready.")
    
    @model.commit_operation
  end
  
  # Generate gradient chart which illustrates insolation calculation results
  def generate_gradient
	prgr_bar=Lss_Progr_Bar.new(@corners4result.length,"|","_",2)
	col_step=200.0/@max_duration # defines the contrast between the darkest and the lightest square of the gradient chart
	@average_duration=0.0
	@corners4result.each_index{|pt_ind|
		prgr_bar.update(pt_ind)
		Sketchup.status_text = "#{$chronoluxStrings.GetString("Generating test face result group in progress")} #{prgr_bar.progr_string}"
		res=@results_array[pt_ind]
		  square_pts=Array.new(@corners4result[pt_ind])
		  res_col=50+(res*col_step).to_i
		  col=[res_col,res_col,res_col]
		  res_face=@test_face_grp.entities.add_face(square_pts)
		  res_face.material=col
		  res_face.back_material=col      
		@average_duration+=res
	}
	Sketchup.status_text = ""
  end
  
  # Generate insolation duration isolines
  def generate_isolines
	# Make an array, which contains indicies of neighboring points for each test point of the test grid.
	# It is necessary, because the face could be of any form (not only rectangular)
	neighb_arr=Array.new(@test_net.length)
	@average_duration=0.0
	prgr_bar=Lss_Progr_Bar.new(@test_net.length,"|","_",2)
	@test_net.each_index{|pt_ind|
		prgr_bar.update(pt_ind)
		Sketchup.status_text = "#{$chronoluxStrings.GetString("Preparing for isolines generation in progress")} #{prgr_bar.progr_string}"
		pt=Geom::Point3d.new(@test_net[pt_ind])
		pt_ind1=pt_ind
		neighb1=nil; neighb2=nil; neighb3=nil
		while pt_ind1<@test_net.length
			chk_pt=Geom::Point3d.new(@test_net[pt_ind1])
			chk_dist=pt.distance(chk_pt)
			if chk_dist==@chk_step_size and neighb1
				neighb2=pt_ind1
			end
			if chk_dist==@chk_step_size and neighb2.nil?
				neighb1=pt_ind1
			end
			if chk_dist==Math.sqrt(@chk_step_size*@chk_step_size+@chk_step_size*@chk_step_size)
				neighb3=pt_ind1
			end
			pt_ind1+=1
		end
		neighb_arr[pt_ind]=[neighb1, neighb2, neighb3]
		@average_duration+=@results_array[pt_ind]
	}
	
	# Iterate through possible durations from minimal possible insolation duration to maximum one
	curr_step=@isoline_step
	step_offset_vec=Geom::Vector3d.new(@selected_face_norm)
	curr_plane=@selected_face_plane
	prgr_bar=Lss_Progr_Bar.new(@max_duration,"|","_",2)
	while curr_step<@max_duration
		prgr_bar.update(curr_step)
	        Sketchup.status_text = "#{$chronoluxStrings.GetString("Isolines generation is in progress")} #{prgr_bar.progr_string}"
		curr_level_grp=@test_face_grp.entities.add_group
		h=$chronoluxStrings.GetString("h")
		m=$chronoluxStrings.GetString("m")
		hr=Time.at(curr_step).utc.hour.to_s; hr="0"+hr if hr.length==1
		mn=Time.at(curr_step).utc.min.to_s; mn="0"+mn if mn.length==1
		grp_name_str="#{hr}#{h}#{mn}#{m}"
		curr_level_grp.name=grp_name_str
		curr_level_grp.layer=@chrono_layers.test_face_layer
		@test_net.each_index{|pt_ind|
			curr_ngb_cnt=0
			neighb_arr[pt_ind].each{|ngb| curr_ngb_cnt+=1 if ngb}
			if curr_ngb_cnt>1
				pt0=Geom::Point3d.new(@test_net[pt_ind])
				pt1=nil; pt2=nil; pt3=nil
				curr_ngbs=neighb_arr[pt_ind]
				pt1=Geom::Point3d.new(@test_net[curr_ngbs[0]]) if curr_ngbs[0]
				pt2=Geom::Point3d.new(@test_net[curr_ngbs[1]]) if curr_ngbs[1]
				pt3=Geom::Point3d.new(@test_net[curr_ngbs[2]]) if curr_ngbs[2]
				res0=@results_array[pt_ind]
				res1=nil; res2=nil; res3=nil
				res1=@results_array[curr_ngbs[0]] if curr_ngbs[0]
				res2=@results_array[curr_ngbs[1]] if curr_ngbs[1]
				res3=@results_array[curr_ngbs[2]] if curr_ngbs[2]
				intpt1=nil; intpt2=nil; intpt3=nil
				if res1
					if curr_step.between?(res0, res1) or curr_step.between?(res1, res0)
						res0_vec=Geom::Vector3d.new(@selected_face_norm); res0_vec.length=res0-curr_step
						res1_vec=Geom::Vector3d.new(@selected_face_norm); res1_vec.length=res1-curr_step
						line = [pt0.offset(res0_vec), pt1.offset(res1_vec)]
						intpt1=Geom.intersect_line_plane(line, curr_plane)
					end
				end
				if res3
					if curr_step.between?(res0, res3) or curr_step.between?(res3, res0)
						res0_vec=Geom::Vector3d.new(@selected_face_norm); res0_vec.length=res0-curr_step
						res3_vec=Geom::Vector3d.new(@selected_face_norm); res3_vec.length=res3-curr_step
						line = [pt0.offset(res0_vec), pt3.offset(res3_vec)]
						intpt2=Geom.intersect_line_plane(line, curr_plane)
					end
				end
				if res3 and res1
					if curr_step.between?(res3, res1) or curr_step.between?(res1, res3)
						res1_vec=Geom::Vector3d.new(@selected_face_norm); res1_vec.length=res1-curr_step
						res3_vec=Geom::Vector3d.new(@selected_face_norm); res3_vec.length=res3-curr_step
						line = [pt1.offset(res1_vec), pt3.offset(res3_vec)]
						intpt3=Geom.intersect_line_plane(line, curr_plane)
					end
				end
				
				curr_level_grp.entities.add_line(intpt1, intpt2) if intpt1 and intpt2
				curr_level_grp.entities.add_line(intpt1, intpt3) if intpt1 and intpt3
				curr_level_grp.entities.add_line(intpt2, intpt3) if intpt3 and intpt2
				
				intpt1=nil; intpt2=nil; intpt3=nil
				if res2
					if curr_step.between?(res0, res2) or curr_step.between?(res2, res0)
						res0_vec=Geom::Vector3d.new(@selected_face_norm); res0_vec.length=res0-curr_step
						res2_vec=Geom::Vector3d.new(@selected_face_norm); res2_vec.length=res2-curr_step
						line = [pt0.offset(res0_vec), pt2.offset(res2_vec)]
						intpt1=Geom.intersect_line_plane(line, curr_plane)
					end
				end
				if res3
					if curr_step.between?(res0, res3) or curr_step.between?(res3, res0)
						res0_vec=Geom::Vector3d.new(@selected_face_norm); res0_vec.length=res0-curr_step
						res3_vec=Geom::Vector3d.new(@selected_face_norm); res3_vec.length=res3-curr_step
						line = [pt0.offset(res0_vec), pt3.offset(res3_vec)]
						intpt2=Geom.intersect_line_plane(line, curr_plane)
					end
				end
				if res3 and res2
					if curr_step.between?(res3, res2) or curr_step.between?(res2, res3)
						res2_vec=Geom::Vector3d.new(@selected_face_norm); res2_vec.length=res2-curr_step
						res3_vec=Geom::Vector3d.new(@selected_face_norm); res3_vec.length=res3-curr_step
						line = [pt2.offset(res2_vec), pt3.offset(res3_vec)]
						intpt3=Geom.intersect_line_plane(line, curr_plane)
					end
				end

				curr_level_grp.entities.add_line(intpt1, intpt2) if intpt1 and intpt2
				curr_level_grp.entities.add_line(intpt1, intpt3) if intpt1 and intpt3
				curr_level_grp.entities.add_line(intpt2, intpt3) if intpt3 and intpt2				
			end
		}
		curr_step+=@isoline_step
	end
	#~ @iso_lines_grp.entities.each{|lev|
		#~ txt_pos=lev.bounds.max
		#~ duration_str=lev.name
		#~ duration_text=lev.entities.add_text(duration_str, txt_pos)
		#~ duration_text.layer=@chrono_layers.test_face_layer
	#~ }
	Sketchup.status_text = ""
  end
  
  # Clicking on a face triggers calculation process
  def onLButtonUp(flags, x, y, view)
    if @calc_in_progress==false
      ph=view.pick_helper
      ph.do_pick x,y
      ph.all_picked.each{|ent|
        if ent.typename=="Face"
          @under_cursor=ent
          verts=ent.outer_loop.vertices
          @face2pick_pnts=Array.new
          verts.each{|vrt| @face2pick_pnts+=[vrt.position]}
          chk_area=@chk_step_size*@chk_step_size
          @pre_calc_pnts_cnt=ent.area/chk_area
          if @pre_calc_pnts_cnt>@test_pnts2draw_lim
            cnt_str=@pre_calc_pnts_cnt.to_i
            warn_str="#{$chronoluxStrings.GetString("Test points count:")} #{cnt_str}\n"
            warn_str+="#{$chronoluxStrings.GetString("Warning: it could take a lot of time to calculate!")}\n"
            warn_str+="#{$chronoluxStrings.GetString("Process clicked face?")}"
            result = UI.messagebox(warn_str, MB_YESNO)
           if result == 6 # Yes
            self.generate_test_net(true)
            @selected_face=ent
	    @selected_face_plane=ent.plane
	    @selected_face_norm=ent.normal
            self.process_test_face(view)
            break
           end
          else
            self.generate_test_net
            @selected_face=ent
	    @selected_face_plane=ent.plane
	    @selected_face_norm=ent.normal
            self.process_test_face(view)
            break
          end
        else
          @face2pick_pnts=Array.new
        end
      }
      
      # Handle the situation when clicked object was not a "Face"
      if @selected_face.nil?
        @under_cursor = ph.best_picked 
        if @under_cursor
          Sketchup.set_status_text("#{@under_cursor.typename} #{$chronoluxStrings.GetString("was clicked.")}")
          view.invalidate
        else
          Sketchup.set_status_text($chronoluxStrings.GetString("Click on face to start testing its surface."))
        end
        @picked_face_pnts=nil
      else
        @picked_face_pnts=Array.new
        @face2pick_pnts.each{|pt| @picked_face_pnts+=[Geom::Point3d.new(pt)]} if @face2pick_pnts
      end
    end
  end
  
  def onCancel(reason, view)
    self.reset(view)
    view.invalidate
    Sketchup.active_model.select_tool nil
  end
  
  def deactivate(view)
    @model.set_attribute("LSS Chronolux Settings", "lss_tool_name", nil)
    self.reset(view)
    view.invalidate
  end
  
  def getInstructorContentDirectory
    dir_path="../../chronolux/instruct/testface"
    return dir_path
  end  
  
end #class Lss_Pick_Test_Face_Tool

if( not file_loaded?("test_face.rb") )
  Lss_Test_Face_Cmds.new
end

#-----------------------------------------------------------------------------
file_loaded("test_face.rb")
