#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ test_pnt_tools.rb ver. 1.0.  08-Oct-11
#~ Test point tools script. It contains commands and tools to calculate insolation duration, svf and wvf at a given point.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

require 'chronolux/sun_scale.rb'

require 'chronolux/layers.rb'

require 'chronolux/test_point.rb'

require 'chronolux/report_maker.rb'

# This is a class, which adds commands for performing insolation and SVF testing at a given test point

class Lss_Chrono_Test_Pnt_Cmds
  def initialize
    @model=Sketchup.active_model
    add_test_pnt_cmd=UI::Command.new($chronoluxStrings.GetString("Add Test Point...")){
      add_test_pnt_tool=Lss_Add_Test_Pnt_Tool.new
      Sketchup.active_model.select_tool add_test_pnt_tool
      dir=add_test_pnt_tool.getInstructorContentDirectory #does not work
    }
    add_test_pnt_cmd.small_icon = "./tb_icons/add_test_pnt_16.png"
    add_test_pnt_cmd.large_icon = "./tb_icons/add_test_pnt_24.png"
    add_test_pnt_cmd.tooltip = $chronoluxStrings.GetString("Click to place a test point on your model")
    $chronoluxToolbar.add_item(add_test_pnt_cmd)
    $chronoluxMenu.add_item(add_test_pnt_cmd)
    
    refresh_test_pnt_cmd=UI::Command.new($chronoluxStrings.GetString("Refresh Test Point(s)")){
      self.refresh_pnts
    }
    refresh_test_pnt_cmd.small_icon = "./tb_icons/refresh_test_pnt_16.png"
    refresh_test_pnt_cmd.large_icon = "./tb_icons/refresh_test_pnt_24.png"
    refresh_test_pnt_cmd.tooltip = $chronoluxStrings.GetString("Click to rebuild rays and/or recalculate SVF and/or WVF")
    $chronoluxToolbar.add_item(refresh_test_pnt_cmd)
    $chronoluxMenu.add_item(refresh_test_pnt_cmd)
    
    set_pnt_no_cmd=UI::Command.new($chronoluxStrings.GetString("Set Current Test Point Number")){
      model=Sketchup.active_model
      old_pnt_no=model.get_attribute "LSS Chronolux Settings", "pnt_no"
      prompts = [$chronoluxStrings.GetString("Current Point Number")]
      defaults = [old_pnt_no]
      input_res = UI.inputbox(prompts, defaults, $chronoluxStrings.GetString("Enter Point Number"))
      if input_res
        model.set_attribute "LSS Chronolux Settings", "pnt_no", input_res[0]
      end
    }
    set_pnt_no_cmd.small_icon = "./tb_icons/set_pnt_no_16.png"
    set_pnt_no_cmd.large_icon = "./tb_icons/set_pnt_no_24.png"
    set_pnt_no_cmd.tooltip = $chronoluxStrings.GetString("Click to set the number of the test point")
    $chronoluxToolbar.add_item(set_pnt_no_cmd)
    $chronoluxMenu.add_item(set_pnt_no_cmd)
    
    props_test_pnt_cmd=UI::Command.new($chronoluxStrings.GetString("Test Point Properties")){
      selection=Sketchup.active_model.selection
      @test_pnt_groups=Array.new
      selection.each{|ent|
        if ent.typename=="Group"
          @test_pnt_groups+=[ent] if ent.name=="LSS Test Point Group"
        end
      }
      if @test_pnt_groups.length==0
        UI.messagebox($chronoluxStrings.GetString("It is necessary to select existing test point(s) to edit its(their) properties."))
      else
        self.build_props_list
        prompts=Array.new
        prompts+=[$chronoluxStrings.GetString("Test point number")]
        prompts+=[$chronoluxStrings.GetString("Short ray length (current units) ")]
        prompts+=[$chronoluxStrings.GetString("Long ray length (current units) ")]
        prompts+=[$chronoluxStrings.GetString("Short ray offset (current units) ")]
        prompts+=[$chronoluxStrings.GetString("Calculate and draw rays")]
        prompts+=[$chronoluxStrings.GetString("Calculate SVF")]
        #~ prompts+=[$chronoluxStrings.GetString("Calculate WVF")]
        prompts+=[$chronoluxStrings.GetString("SVF steps count")]
        #~ prompts+=[$chronoluxStrings.GetString("WVF steps along X")]
        #~ prompts+=[$chronoluxStrings.GetString("WVF steps along Y")]
        prompts+=[$chronoluxStrings.GetString("Show rays checking process")]
        prompts+=[$chronoluxStrings.GetString("Insolated sector representation")]
        prompts+=[$chronoluxStrings.GetString("SVF sphere radius")]
        
        y_str=$chronoluxStrings.GetString("Yes")
        no_str=$chronoluxStrings.GetString("No")
        
        defaults = Array.new
		if not @pnt_no=="..."
          defaults+=[@pnt_no]
        else
          defaults+=["..."]
        end
        if not @short_ray_len=="..."
          defaults+=[Sketchup.format_length(@short_ray_len)]
        else
          defaults+=["..."]
        end
        if not @long_ray_len=="..."
          defaults+=[Sketchup.format_length(@long_ray_len)]
        else
          defaults+=["..."]
        end
        if not @ray_offset=="..."
          defaults+=[Sketchup.format_length(@ray_offset)]
        else
          defaults+=["..."]
        end
        case @calc_rays
          when true
          defaults+=[y_str]
          when false
          defaults+=[no_str]
          when "..."
          defaults+=["..."]
        end
        case @calc_svf
          when true
          defaults+=[y_str]
          when false
          defaults+=[no_str]
          when "..."
          defaults+=["..."]
        end
        #~ case @calc_wvf
          #~ when true
          #~ defaults+=[y_str]
          #~ when false
          #~ defaults+=[no_str]
          #~ when "..."
          #~ defaults+=["..."]
        #~ end
        if @svf_steps_cnt !="..."
          defaults+=[@svf_steps_cnt]
        else
          defaults+=["..."]
        end
        #~ if  @wvf_steps_cnt_x !="..."
          #~ defaults+=[@wvf_steps_cnt_x]
        #~ else
          #~ defaults+=["..."]
        #~ end
        #~ if @wvf_steps_cnt_y !="..."
          #~ defaults+=[@wvf_steps_cnt_y]
        #~ else
          #~ defaults+=["..."]
        #~ end
        case @show_ray_calc
          when true
          defaults+=[y_str]
          when false
          defaults+=[no_str]
          when "..."
          defaults+=["..."]
        end
        if @ins_sector_type != "..."
          defaults+=[@ins_sector_type]
        else
          defaults+=["..."]
        end
        if not @svf_sphere_r=="..."
          defaults+=[Sketchup.format_length(@svf_sphere_r)]
        else
          defaults+=["..."]
        end
        
        list=Array.new
        list+=[""] #@pnt_no
        list+=[""] #@short_ray_len
        list+=[""] #@long_ray_len
        list+=[""] #@ray_offset
        list+=["#{y_str}|#{no_str}|..."] #calc_rays
        list+=["#{y_str}|#{no_str}|..."] #calc_svf
        #~ list+=["#{y_str}|#{no_str}|..."] #calc_wvf
        list+=[""] #@svf_steps_cnt
        #~ list+=[""] #@wvf_steps_cnt_x
        #~ list+=[""] #@wvf_steps_cnt_y
        list+=["#{y_str}|#{no_str}|..."] #@show_ray_calc
        list+=["#{$chronoluxStrings.GetString("Surface")}|#{$chronoluxStrings.GetString("Rays")}|..."] #@ins_sector_type
        list+=[""] #@svf_sphere_r
		
		# Patcth to solve encoding issue
		@model.set_attribute "LSS Chronolux Settings", "y_str", y_str
		y_str=@model.get_attribute "LSS Chronolux Settings", "y_str"
		@model.set_attribute "LSS Chronolux Settings", "no_str", no_str
		no_str=@model.get_attribute "LSS Chronolux Settings", "no_str"
		# End of Patcth to solve encoding issue
		
        input_res = UI.inputbox prompts, defaults, list, $chronoluxStrings.GetString("Test Point Properties")
        if input_res
          #it is necessary to add format checking to fields with numeric values!!!
          ind=0
          @pnt_no=input_res[ind].to_i if not (input_res[ind]=="..." or input_res[ind]==""); ind+=1
          @short_ray_len=Sketchup.parse_length(input_res[ind]) if not (input_res[ind]=="..." or input_res[ind]==""); ind+=1
          @long_ray_len=Sketchup.parse_length(input_res[ind]) if not (input_res[ind]=="..." or input_res[ind]==""); ind+=1
          @ray_offset=Sketchup.parse_length(input_res[ind]) if not (input_res[ind]=="..." or input_res[ind]==""); ind+=1
          if input_res[ind]==y_str
            @calc_rays=true
          else
            @calc_rays=false if input_res[ind]==no_str
          end; ind+=1
          if input_res[ind]==y_str
            @calc_svf=true
          else
            @calc_svf=false if input_res[ind]==no_str
          end; ind+=1
          #~ if input_res[ind]==y_str
            #~ @calc_wvf=true
          #~ else
            #~ @calc_wvf=false if input_res[ind]==no_str
          #~ end; ind+=1
          @svf_steps_cnt=input_res[ind].to_f if not (input_res[ind]=="..." or input_res[ind]==""); ind+=1
          #~ @wvf_steps_cnt_x=input_res[ind].to_f; ind+=1
          #~ @wvf_steps_cnt_y=input_res[ind].to_f; ind+=1
          if input_res[ind]==y_str
            @show_ray_calc=true
          else
            @show_ray_calc=false if input_res[ind]==no_str
          end; ind+=1
          @ins_sector_type=input_res[ind] if not (input_res[ind]=="..." or input_res[ind]==""); ind+=1
          @svf_sphere_r=Sketchup.parse_length(input_res[ind]) if not (input_res[ind]=="..." or input_res[ind]==""); ind+=1

          self.apply_new_properties
          
          result = UI.messagebox($chronoluxStrings.GetString("Is it necessary to refresh changed test points?"), MB_YESNO)
          if result == 6 # Yes
            #refresh
            self.refresh_pnts
          end
        end
      end
    }
    props_test_pnt_cmd.small_icon = "./tb_icons/props_test_pnt_16.png"
    props_test_pnt_cmd.large_icon = "./tb_icons/props_test_pnt_24.png"
    props_test_pnt_cmd.tooltip = $chronoluxStrings.GetString("Click to edit properties of currently selected test point")
    $chronoluxToolbar.add_item(props_test_pnt_cmd)
    $chronoluxMenu.add_item(props_test_pnt_cmd)
    
    #will be implemented in the next version
    #~ pick_win_face_cmd=UI::Command.new($chronoluxStrings.GetString("Pick Window Face")){
      
    #~ }
    #~ pick_win_face_cmd.small_icon = "./tb_icons/pick_win_face_16.png"
    #~ pick_win_face_cmd.large_icon = "./tb_icons/pick_win_face_24.png"
    #~ pick_win_face_cmd.tooltip = $chronoluxStrings.GetString("Click to pick the face, which represents window opening")
    #~ $chronoluxToolbar.add_item(pick_win_face_cmd)
    #~ $chronoluxMenu.add_item(pick_win_face_cmd)
    
    make_rays_cluster_cmd=UI::Command.new($chronoluxStrings.GetString("Make Rays Cluster(s)")){
      self.calc_rays_cpoint
    }
    make_rays_cluster_cmd.small_icon = "./tb_icons/make_rays_cluster_16.png"
    make_rays_cluster_cmd.large_icon = "./tb_icons/make_rays_cluster_24.png"
    make_rays_cluster_cmd.tooltip = $chronoluxStrings.GetString("Click to draw rays cluster of selected test points")
    $chronoluxToolbar.add_item(make_rays_cluster_cmd)
    $chronoluxMenu.add_item(make_rays_cluster_cmd)
    
    calc_svf_cmd=UI::Command.new($chronoluxStrings.GetString("Calculate SVF")){
      self.calc_svf_cpoint
    }
    calc_svf_cmd.small_icon = "./tb_icons/calc_svf_16.png"
    calc_svf_cmd.large_icon = "./tb_icons/calc_svf_24.png"
    calc_svf_cmd.tooltip = $chronoluxStrings.GetString("Click to calculate Sky View Factor")
    $chronoluxToolbar.add_item(calc_svf_cmd)
    $chronoluxMenu.add_item(calc_svf_cmd)
    
    #will be implemented in the next version
    #~ calc_wvf_cmd=UI::Command.new($chronoluxStrings.GetString("Calculate WVF")){
      
    #~ }
    #~ calc_wvf_cmd.small_icon = "./tb_icons/calc_wvf_16.png"
    #~ calc_wvf_cmd.large_icon = "./tb_icons/calc_wvf_24.png"
    #~ calc_wvf_cmd.tooltip = $chronoluxStrings.GetString("Click to calculate Window View Factor")
    #~ $chronoluxToolbar.add_item(calc_wvf_cmd)
    #~ $chronoluxMenu.add_item(calc_wvf_cmd)
    
    list_test_pnts_cmd=UI::Command.new($chronoluxStrings.GetString("List Test Points")){
      test_pnt_groups=Array.new
      selection=Sketchup.active_model.selection
      if selection.length>0
        selection.each{|ent|
          if ent.typename=="Group"
            test_pnt_groups+=[ent] if ent.name=="LSS Test Point Group"
          end
        }
      else
        entities=Sketchup.active_model.entities
        entities.each{|ent|
          if ent.typename=="Group"
            test_pnt_groups+=[ent] if ent.name=="LSS Test Point Group"
          end
        }
      end
      report_inst=Lss_Chrono_Report.new(test_pnt_groups)
      report_inst.build_report_str
      lss_put_report_tool=Lss_Test_Pnts_Report_Tool.new(report_inst.report_str)
      dir=lss_put_report_tool.getInstructorContentDirectory #does not work
      Sketchup.active_model.select_tool(lss_put_report_tool)
    }
    list_test_pnts_cmd.small_icon = "./tb_icons/list_test_pnts_16.png"
    list_test_pnts_cmd.large_icon = "./tb_icons/list_test_pnts_24.png"
    list_test_pnts_cmd.tooltip = $chronoluxStrings.GetString("Click to generate the list of test points")
    $chronoluxToolbar.add_item(list_test_pnts_cmd)
    $chronoluxMenu.add_item(list_test_pnts_cmd)
  end
  
  # Refresh selected test points (erase previous calculation results, then perform insolation testing and/or SVF calculation again)
  def refresh_pnts
    @test_pnt_groups=Array.new
    selection=Sketchup.active_model.selection
    entities=Sketchup.active_model.entities
    if selection.length>0
      selection.each{|ent|
        if ent.typename=="Group"
          @test_pnt_groups+=[ent] if ent.name=="LSS Test Point Group"
        end
      }
    else
      entities.each{|ent|
        if ent.typename=="Group"
          @test_pnt_groups+=[ent] if ent.name=="LSS Test Point Group"
        end
      }
    end
    if @test_pnt_groups.length==0
      UI.messagebox($chronoluxStrings.GetString("It is necessary to select existing test point(s) before refreshing."))
    else
      self.make_test_pt_inst_set
      add_test_pnt_tool=Lss_Add_Test_Pnt_Tool.new
      Sketchup.active_model.select_tool add_test_pnt_tool
      add_test_pnt_tool.test_pnts_set=@test_pnts_set
      view=Sketchup.active_model.active_view
      add_test_pnt_tool.process_test_pnts_set(view)
    end
  end
  
  # Make a set of test points (collect selected groups, which represent test points in an active model into array)
  def make_points_set
    selection=Sketchup.active_model.selection
    entities=Sketchup.active_model.entities
    @cpoints_set=Array.new
    @test_pnt_groups=Array.new
    if selection.length>0
      selection.each{|ent| 
        @cpoints_set+=[ent] if ent.typename=="ConstructionPoint"
        if ent.typename=="Group"
          @test_pnt_groups+=[ent] if ent.name=="LSS Test Point Group"
        end
      }
    end
  end
  
  # Make a set of test point instances (instances of "Lss_Test_Point" class).
  # Erase old calculation results if any.
  def make_test_pt_inst_set
    @test_pnts_set=Array.new
    if @cpoints_set
      if @cpoints_set.length>0
        @cpoints_set.each{|cpoint|
          @test_pt_inst=Lss_Test_Point.new("new")
          @test_pt_inst.create_test_pnt_group(cpoint.position)
          @test_pnts_set+=[@test_pt_inst]
        }
      end
    end
    if @test_pnt_groups.length>0
      @test_pnt_groups.each{|test_pnt_group|
        # Use combination of 2 "key" attributes (point number, and stored calculation time) to find old calculation results
	# and erase them if any.
        pnt_no=test_pnt_group.get_attribute("LSS Test Point Properties", "pnt_no")
        sun_rays_calc_time=test_pnt_group.get_attribute("LSS Test Point Properties", "sun_rays_calc_time")
        svf_calc_time=test_pnt_group.get_attribute("LSS Test Point Properties", "svf_calc_time")
        wvf_calc_time=test_pnt_group.get_attribute("LSS Test Point Properties", "wvf_calc_time")
        self.erase_old_calc_results(pnt_no, sun_rays_calc_time, "rays") if sun_rays_calc_time
        self.erase_old_calc_results(pnt_no, svf_calc_time, "svf") if svf_calc_time
        self.erase_old_calc_results(pnt_no, wvf_calc_time, "wvf") if wvf_calc_time
        @test_pt_inst=Lss_Test_Point.new("existing") # Specify that generating Lss_Test_Point instance is made of existing test point group
        @test_pt_inst.test_pnt_group=test_pnt_group
        @test_pt_inst.read_test_pnt_props
	# The group, which represents test point in the active model could be moved somewhere since its previous position, 
	# so it is necessary to take in account such possible transformation
        test_pnt_group.entities.each{|ent|
          if ent.typename=="ComponentInstance"
            pos=ent.transformation.origin
            pos.transform!(test_pnt_group.transformation)
            @test_pt_inst.position=Geom::Point3d.new(pos)
          end
        } 
        @test_pnts_set+=[@test_pt_inst]
      }
    end
  end
  
  # Enforce insolation calculation at selected test point(s)
  def calc_rays_cpoint
    self.make_points_set
    self.make_test_pt_inst_set
    if @test_pnts_set.length==0
      UI.messagebox($chronoluxStrings.GetString("It is necessary to select construction point(s) and/or existing test points before calling this command."))
    else
      @test_pnts_set.each{|test_pt_inst|
	    pnt_no=test_pt_inst.pnt_no
        test_pt_inst.read_global_settings
		test_pt_inst.pnt_no=pnt_no
        test_pt_inst.calc_rays=true
        test_pt_inst.calc_svf=false
        test_pt_inst.calc_wvf=false
        test_pt_inst.write_test_pnt_props
      }
      add_test_pnt_tool=Lss_Add_Test_Pnt_Tool.new
      Sketchup.active_model.select_tool add_test_pnt_tool
      add_test_pnt_tool.test_pnts_set=@test_pnts_set
      view=Sketchup.active_model.active_view
      add_test_pnt_tool.process_test_pnts_set(view)
    end
  end
  
  # Enforce SVF calculation at selected test point(s)
  def calc_svf_cpoint
    self.make_points_set
    self.make_test_pt_inst_set
    if @test_pnts_set.length==0
      UI.messagebox($chronoluxStrings.GetString("It is necessary to select construction point(s) and/or existing test points before calling this command."))
    else
      @test_pnts_set.each{|test_pt_inst|
		pnt_no=test_pt_inst.pnt_no
        test_pt_inst.read_global_settings
		test_pt_inst.pnt_no=pnt_no
        test_pt_inst.calc_rays=false
        test_pt_inst.calc_svf=true
        test_pt_inst.calc_wvf=false
        test_pt_inst.write_test_pnt_props
      }
      add_test_pnt_tool=Lss_Add_Test_Pnt_Tool.new
      Sketchup.active_model.select_tool add_test_pnt_tool
      add_test_pnt_tool.test_pnts_set=@test_pnts_set
      view=Sketchup.active_model.active_view
      add_test_pnt_tool.process_test_pnts_set(view)
    end
  end
  
  #Build properties list of selected test point(s)
  def build_props_list
    test_pnt_group=@test_pnt_groups[0]
    @pnt_no=test_pnt_group.get_attribute "LSS Test Point Properties", "pnt_no"
    @short_ray_len=test_pnt_group.get_attribute "LSS Test Point Properties", "short_ray_len"
    @long_ray_len=test_pnt_group.get_attribute "LSS Test Point Properties", "long_ray_len"
    @ray_offset=test_pnt_group.get_attribute "LSS Test Point Properties", "ray_offset"
    @start_time=test_pnt_group.get_attribute "LSS Test Point Properties", "start_time"
    @end_time=test_pnt_group.get_attribute "LSS Test Point Properties", "end_time"
    @time_increment=test_pnt_group.get_attribute "LSS Test Point Properties", "time_increment"
    @calc_rays=test_pnt_group.get_attribute "LSS Test Point Properties", "calc_rays"
    @calc_svf=test_pnt_group.get_attribute "LSS Test Point Properties", "calc_svf"
    @calc_wvf=test_pnt_group.get_attribute "LSS Test Point Properties", "calc_wvf"
    @svf_steps_cnt=test_pnt_group.get_attribute "LSS Test Point Properties", "svf_steps_cnt"
    @wvf_steps_cnt_x=test_pnt_group.get_attribute "LSS Test Point Properties", "wvf_steps_cnt_x"
    @wvf_steps_cnt_y=test_pnt_group.get_attribute "LSS Test Point Properties", "wvf_steps_cnt_y"
    @show_ray_calc=test_pnt_group.get_attribute "LSS Test Point Properties", "show_ray_calc"
    @ins_sector_type=test_pnt_group.get_attribute "LSS Test Point Properties", "ins_sector_type"
    @svf_sphere_r=test_pnt_group.get_attribute "LSS Test Point Properties", "svf_sphere_r"
    
    # "..." means different values of the certain property
    @test_pnt_groups.each{|test_pnt_group|
      pnt_no=test_pnt_group.get_attribute "LSS Test Point Properties", "pnt_no"
      @pnt_no="..." if not (pnt_no==@pnt_no)
      short_ray_len=test_pnt_group.get_attribute "LSS Test Point Properties", "short_ray_len"
      @short_ray_len="..." if not (short_ray_len==@short_ray_len)
      long_ray_len=test_pnt_group.get_attribute "LSS Test Point Properties", "long_ray_len"
      @long_ray_len="..." if not (long_ray_len==@long_ray_len)
      ray_offset=test_pnt_group.get_attribute "LSS Test Point Properties", "ray_offset"
      @ray_offset="..." if not (ray_offset==@ray_offset)
      start_time=test_pnt_group.get_attribute "LSS Test Point Properties", "start_time"
      @start_time="..." if not (start_time==@start_time)
      end_time=test_pnt_group.get_attribute "LSS Test Point Properties", "end_time"
      @end_time="..." if not (end_time==@end_time)
      time_increment=test_pnt_group.get_attribute "LSS Test Point Properties", "time_increment"
      @time_increment="..." if not (time_increment==@time_increment)
      calc_rays=test_pnt_group.get_attribute "LSS Test Point Properties", "calc_rays"
      @calc_rays="..." if not (calc_rays==@calc_rays)
      calc_svf=test_pnt_group.get_attribute "LSS Test Point Properties", "calc_svf"
      @calc_svf="..." if not (calc_svf==@calc_svf)
      calc_wvf=test_pnt_group.get_attribute "LSS Test Point Properties", "calc_wvf"
      @calc_wvf="..." if not (calc_wvf==@calc_wvf)
      svf_steps_cnt=test_pnt_group.get_attribute "LSS Test Point Properties", "svf_steps_cnt"
      @svf_steps_cnt="..." if not (svf_steps_cnt==@svf_steps_cnt)
      wvf_steps_cnt_x=test_pnt_group.get_attribute "LSS Test Point Properties", "wvf_steps_cnt_x"
      @wvf_steps_cnt_x="..." if not (wvf_steps_cnt_x==@wvf_steps_cnt_x)
      wvf_steps_cnt_y=test_pnt_group.get_attribute "LSS Test Point Properties", "wvf_steps_cnt_y"
      @wvf_steps_cnt_y="..." if not (wvf_steps_cnt_y==@wvf_steps_cnt_y)
      show_ray_calc=test_pnt_group.get_attribute "LSS Test Point Properties", "show_ray_calc"
      @show_ray_calc="..." if not (show_ray_calc==@show_ray_calc)
      ins_sector_type=test_pnt_group.get_attribute "LSS Test Point Properties", "ins_sector_type"
      @ins_sector_type="..." if not (ins_sector_type==@ins_sector_type)
      svf_sphere_r=test_pnt_group.get_attribute "LSS Test Point Properties", "svf_sphere_r"
      @svf_sphere_r="..." if not (svf_sphere_r==@svf_sphere_r)
    }
  end
  
  # Apply new properties for selected test point(s)
  def apply_new_properties
    ents=Sketchup.active_model.entities
    @test_pnt_groups.each{|test_pnt_group|
      old_pnt_no=test_pnt_group.get_attribute("LSS Test Point Properties", "pnt_no")
      sun_rays_calc_time=test_pnt_group.get_attribute("LSS Test Point Properties", "sun_rays_calc_time")
      svf_calc_time=test_pnt_group.get_attribute("LSS Test Point Properties", "svf_calc_time")
      wvf_calc_time=test_pnt_group.get_attribute("LSS Test Point Properties", "wvf_calc_time")
      
      # If the point number was changed, then it is necessary to change the text representation of it inside the group,
      # which represents test point in an active model
      if not (@pnt_no=="..." or @pnt_no=="")
        test_pnt_group.set_attribute "LSS Test Point Properties", "pnt_no", @pnt_no
        test_pnt_group.entities.each{|ent|
          if ent.typename=="Text"
           ent.text=@pnt_no.to_s if ent.text != @pnt_no.to_s
          end
        }
	
		# If the point number was changed, then it is necessary to change the point number attribute of generated insolation and/or SVF test results.
        # It is due to save associativity between test point and its calculation results (refreshing subsequently uses such associativity for old results erasing). 
        ents.each{|ent|
          if ent.typename=="Group"
            if ent.name=="LSS Sun Rays Group"
              chk_pnt_no=ent.get_attribute("LSS Test Point Properties", "pnt_no")
              chk_time=ent.get_attribute("LSS Test Point Properties", "sun_rays_calc_time")
              ent.set_attribute("LSS Test Point Properties", "pnt_no",@pnt_no) if chk_pnt_no==old_pnt_no and chk_time==sun_rays_calc_time
            end
            if ent.name=="LSS SVF Group"
              chk_pnt_no=ent.get_attribute("LSS Test Point Properties", "pnt_no")
              chk_time=ent.get_attribute("LSS Test Point Properties", "svf_calc_time")
              ent.set_attribute("LSS Test Point Properties", "pnt_no",@pnt_no) if chk_pnt_no==old_pnt_no and chk_time==svf_calc_time
            end
            if ent.name=="LSS WVF Group"
              chk_pnt_no=ent.get_attribute("LSS Test Point Properties", "pnt_no")
              chk_time=ent.get_attribute("LSS Test Point Properties", "wvf_calc_time")
              ent.set_attribute("LSS Test Point Properties", "pnt_no",@pnt_no) if chk_pnt_no==old_pnt_no and chk_time==wvf_calc_time
            end
          end
        }
      end
      
      # Write properties obtained from dialog box to each of selected test point(s) attributes
      test_pnt_group.set_attribute "LSS Test Point Properties", "short_ray_len", @short_ray_len if not (@short_ray_len=="..." or @short_ray_len=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "long_ray_len", @long_ray_len if not (@long_ray_len=="..." or @long_ray_len=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "ray_offset", @ray_offset if not (@ray_offset=="..." or @ray_offset=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "start_time", @start_time if not (@start_time=="..." or @start_time=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "end_time", @end_time if not (@end_time=="..." or @end_time=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "time_increment", @time_increment if not (@time_increment=="..." or @time_increment=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "calc_rays", @calc_rays if not (@calc_rays=="..." or @calc_rays=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "calc_svf", @calc_svf if not (@calc_svf=="..." or @calc_svf=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "calc_wvf", @calc_wvf if not (@calc_wvf=="..." or @calc_svf=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "svf_steps_cnt", @svf_steps_cnt if not (@svf_steps_cnt=="..." or @svf_steps_cnt=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "wvf_steps_cnt_x", @wvf_steps_cnt_x if not (@wvf_steps_cnt_x=="..." or @wvf_steps_cnt_x=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "wvf_steps_cnt_y", @wvf_steps_cnt_y if not (@wvf_steps_cnt_y=="..." or @wvf_steps_cnt_y=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "show_ray_calc", @show_ray_calc if not (@show_ray_calc=="..." or @show_ray_calc=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "ins_sector_type", @ins_sector_type if not (@ins_sector_type=="" or @show_ray_calc=="")
      test_pnt_group.set_attribute "LSS Test Point Properties", "svf_sphere_r", @svf_sphere_r if not (@svf_sphere_r=="..." or @svf_sphere_r=="")
    }
    @test_pnt_groups=nil
  end
  
  # Erase previously generated calculation results (insolation or SVF).
  def erase_old_calc_results(pnt_no, calc_time, calc_type)
    entities=Sketchup.active_model.entities
    case calc_type
      when "rays"
      entities.each{|ent|
        if ent.typename=="Group"
          if ent.name=="LSS Sun Rays Group"
            chk_pnt_no=ent.get_attribute("LSS Test Point Properties", "pnt_no")
            chk_time=ent.get_attribute("LSS Test Point Properties", "sun_rays_calc_time")
            entities.erase_entities(ent) if chk_pnt_no==pnt_no and chk_time==calc_time
          end
        end
      }
      when "svf"
      entities.each{|ent|
        if ent.typename=="Group"
          if ent.name=="LSS SVF Group"
            chk_pnt_no=ent.get_attribute("LSS Test Point Properties", "pnt_no")
            chk_time=ent.get_attribute("LSS Test Point Properties", "svf_calc_time")
            entities.erase_entities(ent) if chk_pnt_no==pnt_no and chk_time==calc_time
          end
        end
      }
      when "wvf"
      entities.each{|ent|
        if ent.typename=="Group"
          if ent.name=="LSS WVF Group"
            chk_pnt_no=ent.get_attribute("LSS Test Point Properties", "pnt_no")
            chk_time=ent.get_attribute("LSS Test Point Properties", "wvf_calc_time")
            entities.erase_entities(ent) if chk_pnt_no==pnt_no and chk_time==calc_time
          end
        end
      }
    end
  end
end #class Lss_Chrono_Test_Pnt_Cmds

# This is a class, which implements a tool for testing insolation duration and SVF at a given point

class Lss_Add_Test_Pnt_Tool
  attr_accessor :test_pnts_set
  attr_accessor :test_pnt_inst
  attr_accessor :lss_tool_name
  def initialize
    @lss_tool_name="Chronolux Add Test Pnt Tool"
    # Make global settings instance to get easy access to each setting through its attr_accessors
	@global_settings=Lss_Chronolux_Init.new 
    @global_settings.read_from_model
    @show_ray_calc=@global_settings.show_ray_calc
    @expr_calc_factor=@global_settings.expr_calc_factor
    @time_increment=@global_settings.time_increment
    
    @calc_rays=@global_settings.calc_rays
    @calc_svf=@global_settings.calc_svf
    @calc_wvf=@global_settings.calc_wvf
    @svf_sphere_r=@global_settings.svf_sphere_r
    @svf_steps_cnt=@global_settings.svf_steps_cnt
    @hemisphere_area=0
    
    @short_ray_len=@global_settings.short_ray_len
    @long_ray_len=@global_settings.long_ray_len
    @ray_offset=@global_settings.ray_offset
    
    @last_time=Time.now
    @model=Sketchup.active_model
    cur_path=Sketchup.find_support_file("add_test_pnt.png", "Plugins/Chronolux/cursors/")
    @add_test_pnt_cur_id=UI.create_cursor(cur_path, 5, 5)
    @ip = nil
    @ip1 = nil
    @rays_arr=Array.new
    @calc_in_progress=false
    
    @lss_sun_scale=Lss_Sun_Scale.new
    @lss_sun_scale.read_sun_scale_from_model
    @sun_scale_times=Array.new(@lss_sun_scale.sun_scale_times)
    @sun_scale_dir_vecs=Array.new(@lss_sun_scale.sun_scale_dir_vecs)
    
    @show_sun_scale=false
    @show_express_calc=false
  end
  
  # Make arrays of times and corresponding sun positions (obtain them from an instance of Lss_Sun_Scale)
  def obtain_sun_scale
    @sun_scale_times=Array.new(@lss_sun_scale.sun_scale_times)
    @sun_scale_dir_vecs=Array.new(@lss_sun_scale.sun_scale_dir_vecs)
    if @sun_scale_times.length==0
      @lss_sun_scale.generate_sun_scale
      @lss_sun_scale.write_sun_scale2model
      @sun_scale_times=Array.new(@lss_sun_scale.sun_scale_times)
      @sun_scale_dir_vecs=Array.new(@lss_sun_scale.sun_scale_dir_vecs)
    end
  end
  
  def activate
    @time_dots=Array.new
    @test_pnts_set=Array.new
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    @model.set_attribute("LSS Chronolux Settings", "lss_tool_name", @lss_tool_name) # programm uses this active_model attribute for help context sensitivity
  end
  
  def onSetCursor
    UI.set_cursor(@add_test_pnt_cur_id)
  end
  
  def onMouseMove(flags, x, y, view)
    @ip1.pick view, x, y
    if( @ip1 != @ip )
      view.invalidate
      @ip.copy! @ip1
      view.tooltip = @ip.tooltip
    end
  end
  
  def draw(view)
    if @ip.valid?
      @ip.draw(view)
      view.draw_points @ip.position, 3, 3,"red"
      # text does not appear if the command was placed once --------
      pnt_no=@model.get_attribute "LSS Chronolux Settings", "pnt_no"
      @tip_str=nil if @calc_in_progress==false #@tip_str=$chronoluxStrings.GetString("The calculation of the test point is in progress now...")
      if @tip_str
	# Show warning at a pointer position if user clicks to add new test point while calculation is in progress
        txt_pt=view.screen_coords(@ip.position) + [10,-20]
        view.draw_text(txt_pt, @tip_str)
        view.draw_text(txt_pt, @tip_str)
      else
	# Show current test point number at a pointer position
        txt_pt=view.screen_coords(@ip.position) + [10,-20]
        view.draw_text(txt_pt, pnt_no.to_s)
        view.draw_text(txt_pt, pnt_no.to_s)
      end
      #--------
      if @calc_rays
        self.draw_sun_scale(view) if @show_sun_scale and @calc_in_progress==false
        self.draw_express_calc(view) if @show_express_calc and @calc_in_progress==false
      end
      if @calc_svf
        self.draw_svf_sphere(view) if @show_express_calc and @calc_in_progress==false
      end
    end
    
    # Show insolation calculation process if it was launched
    if @test_pt_inst
      @calc_in_progress=@test_pt_inst.calc_in_progress
      if @calc_in_progress and @test_pt_inst.position and @test_pt_inst.sun_vec
        sun_vec=Geom::Vector3d.new(@test_pt_inst.sun_vec)
        pt1=Geom::Point3d.new(@test_pt_inst.position)
        len=sun_vec.length
        sun_vec.length=len*2.0
        pt2=pt1.offset(sun_vec)
        
        # text does not appear if the command was placed once ------------
        curr_time=@test_pt_inst.curr_time
        curr_time=curr_time.utc
        hr=curr_time.hour.to_s; hr="0"+hr if hr.length==1
        mn=curr_time.min.to_s; mn="0"+mn if mn.length==1
        time_str="#{hr}:#{mn}"
        txt_pt=view.screen_coords(pt2)
        status = view.drawing_color="black"
        view.draw_text(txt_pt, time_str)
        view.draw_text(txt_pt, time_str)
        #------------
        
        status = view.drawing_color="red"
        view.draw_line(pt1,pt2)
        view.draw_points pt2, 10, 5,"red"
        time_dot=Array.new
        time_dot+=[curr_time]
        time_dot+=[pt2]
        @time_dots+=[time_dot]
        self.draw_time_dots(view)
        self.draw_short_rays(view)
        self.draw_intervals(view)
        self.draw_duration_texts(view)
      else
        @time_dots.clear
      end
      
      # Show SVF calculation process if it was launched
      if @calc_in_progress and @test_pt_inst.position and @test_pt_inst.svf_shadow_mesh
        pnts=@test_pt_inst.svf_shadow_mesh.points
        begin
          view.draw_points(pnts, 6, 2,"black") if pnts
        rescue
          puts "Nothing to draw..."
        end
      end
    end
  end
  
  # Draw SVF hemisphere
  def draw_svf_sphere(view)
    col=Sketchup::Color.new(120, 120, 120)
    col1=Sketchup::Color.new(60, 60, 60)
    lss_svf_sphere=Lss_SVF.new(@ip.position, @svf_sphere_r, @svf_steps_cnt)
    lss_svf_sphere.build_rays_array
    if @hemisphere_area==0
      lss_svf_sphere.calc_hemisphere_area
      @hemisphere_area=lss_svf_sphere.hemisphere_area
    end
    lss_svf_sphere.perform_ray_test
    @svf_sphere_pnts=lss_svf_sphere.sphere_pnts
    @svf_result_pnts=lss_svf_sphere.results_array
    @segm_corners=lss_svf_sphere.segm_corners
    view.draw_points @svf_sphere_pnts, 2, 3, col
    @svf_shadowed_area=0
    @svf_result_pnts.each_index{|ray_ind|
      sph_pnt=Geom::Point3d.new(@svf_sphere_pnts[ray_ind])
      begin
        outer_pnt=Geom::Point3d.new(@svf_result_pnts[ray_ind])
      rescue
        outer_pnt=nil
      end
      if outer_pnt
        status = view.drawing_color=col1
        if ray_ind<@svf_sphere_pnts.length-@svf_steps_cnt
          chk_mod=(ray_ind+@svf_steps_cnt+1).divmod(@svf_steps_cnt)[1]
          if chk_mod!=0
            segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
            segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind+1])
            segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
            segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt+1])
          else
            segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
            segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind-@svf_steps_cnt+1])
            segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
            segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+1])
          end
          view.draw(GL_TRIANGLES, [segm_pnt1,segm_pnt2,segm_pnt3,segm_pnt2,segm_pnt4,segm_pnt3])
          area1=Lss_Geom.new.calc_triangle_area(segm_pnt1,segm_pnt2,segm_pnt3)
          area2=Lss_Geom.new.calc_triangle_area(segm_pnt2,segm_pnt4,segm_pnt3)
          @svf_shadowed_area+=area1+area2
        end
        view.draw_points sph_pnt, 4, 2,"black"
        view.draw_points outer_pnt, 3, 1,"black"
        view.line_stipple = ".."
        view.draw_line(sph_pnt,outer_pnt)
        view.line_stipple = ""
      end
    }
    @svf_clear_area=@hemisphere_area-@svf_shadowed_area
    @svf=(10000.0*@svf_clear_area/@hemisphere_area).round/100.0
    svf_str="SVF=#{@svf}%"
    txt_pt=view.screen_coords(@svf_sphere_pnts[0])+[10,10]
    view.draw_text(txt_pt, svf_str)
    view.draw_text(txt_pt, svf_str)
  end
  
  # Draw time dots while demonstrating insolation calculation process
  def draw_time_dots(view)
    col=Sketchup::Color.new(120, 120, 120)
    @time_dots.each{|td|
      curr_time=td[0]
      pt=Geom::Point3d.new(td[1])
      chk_mod=curr_time.min.divmod(15)[1]
      view.draw_points pt, 3, 1,col if chk_mod==0 and not (curr_time.min==0)
      view.draw_points pt, 6, 6,"red" if curr_time.min==0
    }
  end
  
  # Draw rays, which represent insolation presense (while final insolation calculation at a given test point processes)
  def draw_short_rays(view)
    pnts_arr=Array.new(@test_pt_inst.ray_end_pnt_array)
    pt1=Geom::Point3d.new(@test_pt_inst.position)
    pnts_arr.reverse!
    pnts_arr.each_index{|ind|
      pt=pnts_arr[ind]
      if ind<120
        col=Sketchup::Color.new(ind, ind, ind)
      else
        col=Sketchup::Color.new(120, 120, 120)
      end
      status = view.drawing_color=col
      offset_vec=pt1.vector_to(pt)
      offset_vec.length=@test_pt_inst.ray_offset
      pt2=pt1.offset(offset_vec)
      view.draw_line(pt2,pt)
      status = view.drawing_color="black"
    }
  end
  
  # Draw intervals' bounds times and bound long rays (while final insolation calculation at a given test point processes)
  def draw_intervals(view)
    status = view.drawing_color="black"
    @ins_intervals_array=Array.new(@test_pt_inst.ins_intervals_array)
    if @ins_intervals_array.length>0
      int_bnd=0
      pt1=Geom::Point3d.new(@test_pt_inst.position)
      while int_bnd<@ins_intervals_array.length
        curr_int=@ins_intervals_array[int_bnd]
        int_time=curr_int[0].utc
        sun_vec=curr_int[1]
        sun_vec.length=@test_pt_inst.long_ray_len
        pt2=pt1.offset(sun_vec)
        view.draw_line(pt1,pt2)
        hr=int_time.hour.to_s; hr="0"+hr if hr.length==1
        mn=int_time.min.to_s; mn="0"+mn if mn.length==1
        time_str="#{hr}:#{mn}"
        txt_pt=view.screen_coords(pt2)
        status = view.drawing_color="black"
        view.draw_text(txt_pt, time_str)
        view.draw_text(txt_pt, time_str)
        
        int_bnd+=1
      end
    end
  end
  
  # Draw intervals' durations (while final insolation calculation at a given test point processes)
  def draw_duration_texts(view)
    int_bound_no=0
    if @ins_intervals_array.length>0
      pt1=Geom::Point3d.new(@test_pt_inst.position)
      while int_bound_no<@ins_intervals_array.length
        int_bound_st=Array.new(@ins_intervals_array[int_bound_no])
        if int_bound_no+1<@ins_intervals_array.length
          int_bound_end=Array.new(@ins_intervals_array[int_bound_no+1])
        else
          int_bound_end=Array.new
          int_bound_end+=[@test_pt_inst.curr_time]
          int_bound_end+=[@test_pt_inst.sun_vec]
        end
        int_duration1=Time.at(int_bound_end[0]-int_bound_st[0])
        int_duration=int_duration1.utc
        txt_pos_offset_vec=Geom.linear_combination(0.5, int_bound_st[1], 0.5, int_bound_end[1])
        txt_pos_offset_vec.length=1.1*@test_pt_inst.short_ray_len
        txt_pos=pt1.offset(txt_pos_offset_vec)
        h=$chronoluxStrings.GetString("h")
        m=$chronoluxStrings.GetString("m")
        hr=int_duration.hour.to_s; hr="0"+hr if hr.length==1
        mn=int_duration.min.to_s; mn="0"+mn if mn.length==1
        duration_str="#{hr}#{h}#{mn}#{m}"
        txt_pt=view.screen_coords(txt_pos)
        status = view.drawing_color="black"
        view.draw_text(txt_pt, duration_str)
        view.draw_text(txt_pt, duration_str)
        int_bound_no+=2
      end
    end
  end
  
  # Draw sun scale (it helps to find out insolation presense time intervals if express insolation calculation mode is enabled)
  def draw_sun_scale(view)
    col=Sketchup::Color.new(120, 120, 120)
    @sun_scale_times.each_index{|ray_ind|
      curr_time=@sun_scale_times[ray_ind]
      sun_dir_vec=@sun_scale_dir_vecs[ray_ind]
      long_ray=Geom::Vector3d.new(sun_dir_vec)
      long_ray.length=@long_ray_len
      short_ray=Geom::Vector3d.new(sun_dir_vec)
      short_ray.length=@short_ray_len
      ray_offset=Geom::Vector3d.new(sun_dir_vec)
      ray_offset.length=@ray_offset
      chk_mod=curr_time.min.divmod(15)[1]
      pt_long=@ip.position.offset(long_ray)
      pt_short=@ip.position.offset(short_ray)
      pt_offset=@ip.position.offset(ray_offset)
      if chk_mod==0 and not (curr_time.min==0)
        status = view.drawing_color=col
        view.draw_line(pt_offset,pt_long)
        view.draw_points pt_long, 3, 1,col
      end
      if curr_time.min==0
        status = view.drawing_color="black"
        view.draw_line(@ip.position,pt_long)
        view.draw_points pt_long, 6, 6,"red"
        
        hr=curr_time.hour.to_s; hr="0"+hr if hr.length==1
        mn=curr_time.min.to_s; mn="0"+mn if mn.length==1
        time_str="#{hr}:#{mn}"
        txt_pt=view.screen_coords(pt_long)+[0,-25]
        status = view.drawing_color="black"
        view.draw_text(txt_pt, time_str)
        view.draw_text(txt_pt, time_str) #it is necessary to duplicate (one code line does not work)
      end
    }
  end
  
  # Draw rays, which represent insolation presence and absence in real-time, while moving a
  # pointer over an active model (express calculation mode has to be enabled)
  def draw_express_calc(view)
    short_rays_ends=Array.new
    short_rays_starts=Array.new
    shadow_rays_ends=Array.new
    col=Sketchup::Color.new("red")
    @sun_scale_times.each_index{|ray_ind|
      curr_time=@sun_scale_times[ray_ind]
      sun_dir_vec=@sun_scale_dir_vecs[ray_ind]
      long_ray=Geom::Vector3d.new(sun_dir_vec)
      long_ray.length=@long_ray_len
      short_ray=Geom::Vector3d.new(sun_dir_vec)
      short_ray.length=@short_ray_len
      ray_offset=Geom::Vector3d.new(sun_dir_vec)
      ray_offset.length=@ray_offset
      
      # Express calculation performs less detailed insolation testing (to speed up the process),
      # @expr_calc_factor determines how less detailed express calculation is.
      chk_mod=ray_ind.divmod(@expr_calc_factor)[1]
      if chk_mod==0
        pt=@ip.position
        sun_ray=[pt,Geom::Vector3d.new(sun_dir_vec)]
        intersect_res=Lss_Raytest.new(sun_ray).test
        if intersect_res.nil?
          short_rays_ends+=[pt.offset(short_ray)]
          short_rays_starts+=[pt.offset(ray_offset)]
        else
          shadow_rays_ends+=[intersect_res[0]]
        end
      end
    }
    short_rays_starts.each_index{|ray_ind|
      pt1=short_rays_starts[ray_ind]
      pt2=short_rays_ends[ray_ind]
      status = view.drawing_color=col
      view.draw_line(pt1,pt2)
    }
    duration_val=(short_rays_starts.length-1)*@expr_calc_factor*@time_increment
    duration_val=0 if duration_val<0
    approx_duration=Time.at(duration_val).utc #approximation level depends on @expr_calc_factor
    h=$chronoluxStrings.GetString("h")
    m=$chronoluxStrings.GetString("m")
    hr=approx_duration.hour.to_s; hr="0"+hr if hr.length==1
    mn=approx_duration.min.to_s; mn="0"+mn if mn.length==1
    approx_duration_str=$chronoluxStrings.GetString("approximate duration: ")+"#{hr}#{h}#{mn}#{m}"
    txt_pt=view.screen_coords(@ip.position)+[15,10,0]
    status = view.drawing_color="red"
    view.draw_text(txt_pt, approx_duration_str)
    view.draw_text(txt_pt, approx_duration_str) #it is necessary to duplicate (one code line does not work)
    shadow_rays_ends.each{|end_pt|
      status = view.drawing_color="black"
      status = view.draw_points end_pt, 5, 2, "black"
      ray_offset=@ip.position.vector_to(end_pt)
      ray_offset.length=@ray_offset
      pt=@ip.position.offset(ray_offset)
      view.line_stipple = ".."
      view.draw_line(pt,end_pt)
      view.line_stipple = ""
    }
  end
  
  def reset(view)
    @ip.clear
    @ip1.clear
    if( view )
      view.tooltip = nil
      view.invalidate
    end
  end
  
  def deactivate(view)
    self.reset(view)
    @model.set_attribute("LSS Chronolux Settings", "lss_tool_name", nil)
  end
  
  # Add new group in an active model after single mouse-click
  def onLButtonUp(flags, x, y, view)
    @ip.pick view, x, y
    click_delay=(Time.now-@last_time)
    if click_delay>0.44 #to distinguish from double-click 0.44 is an average double-click "duration"
      if @calc_in_progress
        #put some tooltips here telling that it's better to wait until calculation process will finish
        @tip_str=$chronoluxStrings.GetString("The calculation of the test point is in progress now...")
      else
        @test_pt_inst=Lss_Test_Point.new("new")
        @test_pt_inst.create_test_pnt_group(@ip.position)
        @test_pnts_set+=[@test_pt_inst]
      end
    end
    @last_time=Time.now
  end
  
  # Launch calculation immediately at a pointer position after double-click
  def onLButtonDoubleClick(flags, x, y, view)
    #i need to add a flag to check if there were double click(s) to inform after deactivation, that some points were already processed
    #and skip processed points after deactivation if any
    #and do not show question messagebox if all test points were processed
    @ip.pick view, x, y
    if @calc_in_progress
      #put some tooltips here telling that it's better to wait until calculation process will finish
      @tip_str=$chronoluxStrings.GetString("The calculation of the test point is in progress now...")
    else
      if @test_pt_inst
        #the instance @test_pt_inst was already created during onLButtonUp (the first click of double-click)
        #~ so it is necessary only to calculate_insolation_duration
        self.start_calc_test_pnt
      else
        @test_pt_inst=Lss_Test_Point.new("new")
        @test_pt_inst.create_test_pnt_group(@ip.position)
        @test_pnts_set+=[@test_pt_inst]
        self.start_calc_test_pnt
      end
    end
  end
  
  # Handle some hot-key strokes while the tool is active
  def onKeyUp(key, repeat, flags, view)
    
    # Delete the last added or even calculated test point and decrease back current test point number
    if key==VK_DELETE
      if @test_pnts_set.length>0
        curr_inst=@test_pnts_set.last
        ents_to_erase=Array.new
        ents_to_erase+=[curr_inst.test_pnt_group] if curr_inst.test_pnt_group
        ents_to_erase+=[curr_inst.sun_rays_group] if curr_inst.sun_rays_group
        ents_to_erase+=[curr_inst.svf_group] if curr_inst.svf_group
        entities=Sketchup.active_model.active_entities
        entities.erase_entities(ents_to_erase)
        @test_pnts_set.delete_at(@test_pnts_set.length-1)
        pnt_no=Sketchup.active_model.get_attribute "LSS Chronolux Settings", "pnt_no"
        value = Sketchup.active_model.set_attribute "LSS Chronolux Settings", "pnt_no", pnt_no-1
      end
    end
    
    # Show sun scale
    if key==VK_INSERT
      if @show_sun_scale
        @show_sun_scale=false
      else
        @show_sun_scale=true
      end
      if @sun_scale_times.length==0 and @show_sun_scale
        self.obtain_sun_scale
      end
      view.invalidate
    end
    
    # Toggle "Express calculation" mode
    if key==VK_CONTROL
      if @show_express_calc
        @show_express_calc=false
      else
        @show_express_calc=true
      end
      
      # If there is no sun scale data in the current active model, then generate the sun scale
      if @sun_scale_times.length==0 and @show_express_calc and @calc_rays
        self.obtain_sun_scale
      end
      view.invalidate
    end
  end
  
  # Start calculation at a given point (usualy launches after double-click at its position, while the tool is active)
  def start_calc_test_pnt
    @calc_rays=@test_pt_inst.calc_rays
    @calc_svf=@test_pt_inst.calc_svf
    if @calc_rays and @calc_svf==false
      @test_pt_inst.calculate_insolation_duration(@show_ray_calc)
    end
    if @calc_svf and @calc_rays==false
      @test_pt_inst.calc_n_draw_svf(@test_pt_inst.position, @test_pt_inst.svf_sphere_r, @test_pt_inst.svf_steps_cnt, @show_ray_calc)
    end
	
	# patch
    # This is a temporary patch, it is necessary to be replaced by more sophisticated timers logic
	# The point is that it's not possible to show calculation process of SVF and insolation at once
	# (there is only one timer) so if insolation and SVF are to be calculated, then programm does not
	# show calculation process (i.e. does not use timer).
    if @calc_rays and @calc_svf
      @calc_in_progress=true
      @test_pt_inst.calculate_insolation_duration(false)
      @test_pt_inst.calc_n_draw_svf(@test_pt_inst.position, @test_pt_inst.svf_sphere_r, @test_pt_inst.svf_steps_cnt, false)
      @calc_in_progress=false
      @test_pt_inst.calc_in_progress=false
    end
    # end of patch
  end
  
  # Process a set of entered test points (usually at the end of tool processing)
  def process_test_pnts_set(view)
    test_pnt_grps_set=Array.new
    if @show_ray_calc
      curr_pt_no=0
      @test_pt_inst=@test_pnts_set[curr_pt_no]
      @calc_in_progress=true
      test_pnt_group=@test_pt_inst.test_pnt_group
      test_pnt_grps_set+=[test_pnt_group]
      if @test_pt_inst.is_calculated==false
        self.start_calc_test_pnt
      end
      pnts_timer_id=UI.start_timer(0.01,true){
        if curr_pt_no==@test_pnts_set.length-1 and @test_pt_inst.is_calculated
          report_inst=Lss_Chrono_Report.new(test_pnt_grps_set)
          report_inst.build_report_str
          report_inst.report2console
          pnts_timer_id=UI.stop_timer(pnts_timer_id)
          self.reset(view)
          Sketchup.active_model.select_tool nil
        end
        if @test_pt_inst.is_calculated
          curr_pt_no+=1
          @test_pt_inst=@test_pnts_set[curr_pt_no]
          if @test_pt_inst
            test_pnt_group=@test_pt_inst.test_pnt_group
            test_pnt_grps_set+=[test_pnt_group]
            if @test_pt_inst.is_calculated==false
              self.start_calc_test_pnt
            end
          end
        end
      }
    else
      prgr_bar=Lss_Progr_Bar.new(@test_pnts_set.length,"|","_",2)
      @test_pnts_set.each_index{|ind|
        prgr_bar.update(ind)
        result = Sketchup.status_text = "#{$chronoluxStrings.GetString("Processing test point:")} #{ind} #{$chronoluxStrings.GetString("of")} #{@test_pnts_set.length} #{prgr_bar.progr_string}"
        @test_pt_inst=@test_pnts_set[ind]
        test_pnt_group=@test_pt_inst.test_pnt_group
        test_pnt_grps_set+=[test_pnt_group]
        if @test_pt_inst.is_calculated==false
          @calc_rays=@test_pt_inst.calc_rays
          @calc_svf=@test_pt_inst.calc_svf
          if @calc_rays
            @test_pt_inst.calculate_insolation_duration(@show_ray_calc)
          end
          if @calc_svf
            @test_pt_inst.calc_n_draw_svf(@test_pt_inst.position, @test_pt_inst.svf_sphere_r, @test_pt_inst.svf_steps_cnt, @show_ray_calc)
          end
        end
      }
      report_inst=Lss_Chrono_Report.new(test_pnt_grps_set)
      report_inst.build_report_str
      report_inst.report2console
    end
  end
  
  def onCancel(reason, view)
    if @test_pt_inst
      @test_pt_inst.stop_calc=true if @test_pt_inst.calc_in_progress
    end
    if @test_pnts_set.length==0
      self.reset(view)
      Sketchup.active_model.select_tool nil
    end
    ready_cnt=0
    @test_pnts_set.each{|test_pt_inst|
      ready_cnt+=1 if test_pt_inst.is_calculated
    }
    if @test_pnts_set.length==ready_cnt
      self.reset(view)
      Sketchup.active_model.select_tool nil
    end
	
	# If there were some test points entered, then ask if its necessary to process them
    if @test_pnts_set.length>0
      case ready_cnt
        when 0
        message_str="#{$chronoluxStrings.GetString("Is it necessary to process entered test points")} (#{@test_pnts_set.length.to_s} #{$chronoluxStrings.GetString("points were entered")})?"
        when @test_pnts_set.length
        puts "#{ready_cnt.to_s} #{$chronoluxStrings.GetString("test point(s) was(were) successfully processed.")}"
        when 0..@test_pnts_set.length
        message_str="#{$chronoluxStrings.GetString("Is it necessary to process entered test points")} (#{(@test_pnts_set.length-ready_cnt).to_s} #{$chronoluxStrings.GetString("points were not processed")})?"
      end
      result=nil
      result = UI.messagebox message_str, MB_YESNO if (ready_cnt<@test_pnts_set.length)
      if (result==6)
        self.process_test_pnts_set(view)
      else
        self.reset(view)
        Sketchup.active_model.select_tool nil
      end
    end
    if @show_ray_calc==false
      self.reset(view)
      Sketchup.active_model.select_tool nil
    end
  end

  def enableVCB?
    return true
  end
  
  # Use entered from the keyboard number (while the tool is active) as a current test point number
  def onUserText(text, view)
    @pnt_no=text.to_i
    @pnt_no=1 if @pnt_no==0
    @model.set_attribute "LSS Chronolux Settings", "pnt_no", @pnt_no
  end
  
  # Tool context menu
  def getMenu(menu)
    set_pnt_no_cmd=UI::Command.new($chronoluxStrings.GetString("Set Current Test Point Number")){
      model=Sketchup.active_model
      old_pnt_no=model.get_attribute "LSS Chronolux Settings", "pnt_no"
      prompts = [$chronoluxStrings.GetString("Current Point Number")]
      defaults = [1]
      input_res = UI.inputbox prompts, defaults, "#{$chronoluxStrings.GetString("Enter Point Number")} (it is #{old_pnt_no.to_s} now)"
      if input_res
        model.set_attribute "LSS Chronolux Settings", "pnt_no", input_res[0]
      end
    }
    menu.add_item(set_pnt_no_cmd)
    
    menu.add_item($chronoluxStrings.GetString("Choose what to calculate")) {
      prompts = [$chronoluxStrings.GetString("Choose what to calculate")]
      defaults = [$chronoluxStrings.GetString("Insolation")]
      list=["#{$chronoluxStrings.GetString("Insolation")}|#{$chronoluxStrings.GetString("SVF")}|#{$chronoluxStrings.GetString("Both")}"]
      input_res = UI.inputbox prompts, defaults, list, $chronoluxStrings.GetString("Choose what to calculate")
	  # Patch to solve encoding issue
	  @model.set_attribute "LSS Chronolux Settings", "Insolation", $chronoluxStrings.GetString("Insolation")
	  @model.set_attribute "LSS Chronolux Settings", "SVF", $chronoluxStrings.GetString("SVF")
	  @model.set_attribute "LSS Chronolux Settings", "Both", $chronoluxStrings.GetString("Both")
	  ins_str=@model.get_attribute "LSS Chronolux Settings", "Insolation"
	  svf_str=@model.get_attribute "LSS Chronolux Settings", "SVF"
	  both_str=@model.get_attribute "LSS Chronolux Settings", "Both"
	  # End of patch to solve encoding issue
      if input_res
        case input_res[0]
          when ins_str # Patch to solve encoding issue
          @calc_rays=true
          @calc_svf=false
          when svf_str # Patch to solve encoding issue
          @calc_rays=false
          @calc_svf=true
          when both_str # Patch to solve encoding issue
          @calc_rays=true
          @calc_svf=true
        end
        @model.set_attribute "LSS Chronolux Settings", "calc_rays", @calc_rays
        @model.set_attribute "LSS Chronolux Settings", "calc_svf", @calc_svf
      end
    }
  end
  
  def getInstructorContentDirectory
    dir_path="../../chronolux/instruct/addtstpt"
    return dir_path
  end
 
end #class Lss_Add_Test_Pnt_Tool

# This is a class, which implements a tool for placing a report of processed test points into a specified place in an active model

class Lss_Test_Pnts_Report_Tool
  attr_accessor :lss_tool_name
  def initialize(report_str) # Use report string (list of processed test points results) as an argument
    @lss_tool_name="Chronolux List Test Points Tool"
    @report_str=report_str.gsub(/\t/,"  | ") # "/..../" is a regular expression used to match a pattern against strings
    @model=Sketchup.active_model
    cur_path=Sketchup.find_support_file("put_pnts_list.png", "Plugins/Chronolux/cursors/")
    @add_test_pnt_cur_id=UI.create_cursor(cur_path,0,0)
    @ip = nil
    @ip1 = nil
  end
  
  def activate
    @model.set_attribute("LSS Chronolux Settings", "lss_tool_name", @lss_tool_name) # programm uses this active_model attribute for help context sensitivity
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
  end
  
  def onSetCursor
    UI.set_cursor(@add_test_pnt_cur_id)
  end
  
  def onMouseMove(flags, x, y, view)
    @ip1.pick view, x, y
    if( @ip1 != @ip )
      view.invalidate
      @ip.copy! @ip1
      view.tooltip = @ip.tooltip
    end
  end
  
  def draw(view)
    if @ip.valid? and @report_str
      @ip.draw(view)
      # text does not appear if the command was placed once --------
      txt_pt=view.screen_coords(@ip.position)
      view.draw_text(txt_pt, " ")
      view.draw_text(txt_pt, @report_str)
      #--------
    end
  end
  
  # Place 2d text object at a given point after single click
  def onLButtonUp(flags, x, y, view)
    @ip.pick view, x, y
    @model.entities.add_text(@report_str,@ip.position)
  end
  
  def reset(view)
    @report_str=nil
    @ip.clear
    @ip1.clear
    if( view )
      view.tooltip = nil
      view.invalidate
    end
  end

  
  def onCancel(reason, view)
    self.reset(view)
    Sketchup.active_model.select_tool(nil)
  end
  
  def deactivate(view)
    @model.set_attribute("LSS Chronolux Settings", "lss_tool_name", nil)
  end
  
  def getInstructorContentDirectory
    dir_path="../../chronolux/instruct/pntslist"
    return dir_path
  end
  
end #class Lss_Test_Pnts_Report_Tool

if( not file_loaded?("test_pnt_tools.rb") )
  Lss_Chrono_Test_Pnt_Cmds.new
end

#-----------------------------------------------------------------------------
file_loaded("test_pnt_tools.rb")