#~ '(C) 2010, Links System Software
#~ 'Feedback information
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru (search this e-mail to add skype contact)
#~ 'icq: 328-958-369

#~ test_pnt.rb ver. 1.0.  08-Oct-11
#~ Test point script. It contains methods to calculate and draw calculation results at a given point

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

require 'chronolux/layers.rb'

# This is a class, which implements calculation of insolation duration and SVF at a given point.

class Lss_Test_Point
  attr_accessor :calc_in_progress
  attr_accessor :sun_vec
  attr_accessor :position
  attr_accessor :curr_time
  attr_accessor :test_pnt_group
  attr_accessor :stop_calc
  attr_accessor :ray_end_pnt_array
  attr_accessor :ins_intervals_array
  attr_accessor :is_calculated
  attr_accessor :test_pnt_group
  attr_accessor :sun_rays_group
  attr_accessor :svf_group
  
  attr_accessor :svf_shadow_mesh
  
  attr_accessor :pnt_no
  attr_accessor :short_ray_len
  attr_accessor :long_ray_len
  attr_accessor :ray_offset
  attr_accessor :start_time
  attr_accessor :end_time
  attr_accessor :time_increment
  attr_accessor :calc_rays
  attr_accessor :calc_svf
  attr_accessor :calc_wvf
  attr_accessor :svf_steps_cnt
  attr_accessor :wvf_steps_cnt_x
  attr_accessor :wvf_steps_cnt_y
  attr_accessor :show_ray_calc
  attr_accessor :expr_calc_factor
  attr_accessor :ins_sector_type
  attr_accessor :svf_sphere_r
  attr_accessor :expr_calc_2dscl
  attr_accessor :long_ray_len2d
  attr_accessor :svf_sphere_r2d
  
  attr_accessor :ins_sector_type
  
  def initialize(test_pnt_type) # Two types are possible "new" and "existing"
    @is_calculated=false
    @chrono_layers=Lss_Chrono_Layers.new
    @ray_end_pnt_array=Array.new
    @ray_time_array=Array.new
    @stop_calc=false
    @model=Sketchup.active_model
    @entities=Sketchup.active_model.active_entities
    @test_pnt_position=nil
    @ins_intervals_array=Array.new
    @total_ins_duration=0.0 #as float (it is necessary to use Time.at(@total_ins_duration).utc
    @svf=0.0
    @wvf=0.0
    @calc_in_progress=false
    self.read_global_settings
    @sun_rays_calc_time=Time.now
    @svf_calc_time=Time.now
    @wvf_calc_time=Time.now
  end
  
  # Clear old insolation duration intervals from test point group attributes if any
  # (programm stores insolation calculation results within attributes of a group, which
  # represents a test point, and uses stored information to generate a report of processed
  # test points calculation results subsequently)
  def clear_old_intervals
    props_dict=@test_pnt_group.attribute_dictionary("LSS Test Point Properties")
    if props_dict
      props_dict.each_key{|key|
        if key.split[0]=="ins_interval"
          props_dict.delete_key(key)
        end
      }
    end
  end
  
  def read_global_settings
    @global_settings=Lss_Chronolux_Init.new
    @global_settings.read_from_model
    
    @pnt_no=@global_settings.pnt_no
    @short_ray_len=@global_settings.short_ray_len
    @long_ray_len=@global_settings.long_ray_len
    @ray_offset=@global_settings.ray_offset
    @start_time=@global_settings.start_time
    @end_time=@global_settings.end_time
    @time_increment=@global_settings.time_increment
    @calc_rays=@global_settings.calc_rays
    @calc_svf=@global_settings.calc_svf
    @calc_wvf=@global_settings.calc_wvf
    @svf_steps_cnt=@global_settings.svf_steps_cnt
    @wvf_steps_cnt_x=@global_settings.wvf_steps_cnt_x
    @wvf_steps_cnt_y=@global_settings.wvf_steps_cnt_y
    @show_ray_calc=@global_settings.show_ray_calc
    @expr_calc_factor=@global_settings.expr_calc_factor
    @ins_sector_type=@global_settings.ins_sector_type
    @svf_sphere_r=@global_settings.svf_sphere_r
    @expr_calc_2dscl=@global_settings.expr_calc_2dscl
  end
  
  def read_test_pnt_props
    @pnt_no=@test_pnt_group.get_attribute "LSS Test Point Properties", "pnt_no"
    @short_ray_len=@test_pnt_group.get_attribute "LSS Test Point Properties", "short_ray_len"
    @long_ray_len=@test_pnt_group.get_attribute "LSS Test Point Properties", "long_ray_len"
    @ray_offset=@test_pnt_group.get_attribute "LSS Test Point Properties", "ray_offset"
    @start_time=@test_pnt_group.get_attribute "LSS Test Point Properties", "start_time"
    @end_time=@test_pnt_group.get_attribute "LSS Test Point Properties", "end_time"
    @time_increment=@test_pnt_group.get_attribute "LSS Test Point Properties", "time_increment"
    @calc_rays=@test_pnt_group.get_attribute "LSS Test Point Properties", "calc_rays"
    @calc_svf=@test_pnt_group.get_attribute "LSS Test Point Properties", "calc_svf"
    @calc_wvf=@test_pnt_group.get_attribute "LSS Test Point Properties", "calc_wvf"
    @svf_steps_cnt=@test_pnt_group.get_attribute "LSS Test Point Properties", "svf_steps_cnt"
    @wvf_steps_cnt_x=@test_pnt_group.get_attribute "LSS Test Point Properties", "wvf_steps_cnt_x"
    @wvf_steps_cnt_y=@test_pnt_group.get_attribute "LSS Test Point Properties", "wvf_steps_cnt_y"
    @show_ray_calc=@test_pnt_group.get_attribute "LSS Test Point Properties", "show_ray_calc"
    @ins_sector_type=@test_pnt_group.get_attribute "LSS Test Point Properties", "ins_sector_type"
    @svf_sphere_r=@test_pnt_group.get_attribute "LSS Test Point Properties", "svf_sphere_r"
  end
  
  def write_test_pnt_props
    @test_pnt_group.set_attribute "LSS Test Point Properties", "pnt_no", @pnt_no
    @test_pnt_group.set_attribute "LSS Test Point Properties", "short_ray_len", @short_ray_len
    @test_pnt_group.set_attribute "LSS Test Point Properties", "long_ray_len", @long_ray_len
    @test_pnt_group.set_attribute "LSS Test Point Properties", "ray_offset", @ray_offset
    @test_pnt_group.set_attribute "LSS Test Point Properties", "start_time", @start_time
    @test_pnt_group.set_attribute "LSS Test Point Properties", "end_time", @end_time
    @test_pnt_group.set_attribute "LSS Test Point Properties", "time_increment", @time_increment
    @test_pnt_group.set_attribute "LSS Test Point Properties", "calc_rays", @calc_rays
    @test_pnt_group.set_attribute "LSS Test Point Properties", "calc_svf", @calc_svf
    @test_pnt_group.set_attribute "LSS Test Point Properties", "calc_wvf", @calc_wvf
    @test_pnt_group.set_attribute "LSS Test Point Properties", "svf_steps_cnt", @svf_steps_cnt
    @test_pnt_group.set_attribute "LSS Test Point Properties", "wvf_steps_cnt_x", @wvf_steps_cnt_x
    @test_pnt_group.set_attribute "LSS Test Point Properties", "wvf_steps_cnt_y", @wvf_steps_cnt_y
    @test_pnt_group.set_attribute "LSS Test Point Properties", "show_ray_calc", @show_ray_calc
    @test_pnt_group.set_attribute "LSS Test Point Properties", "ins_sector_type", @ins_sector_type
    @test_pnt_group.set_attribute "LSS Test Point Properties", "svf_sphere_r", @svf_sphere_r
  end
  
  # Create the group wich represents a test point at a given position in an active model
  def create_test_pnt_group(pos)
    @position=Geom::Point3d.new(pos)
    @test_pnt_group=@entities.add_group
    @test_pnt_group.name="LSS Test Point Group"
    @test_pnt_group.layer=@chrono_layers.test_pnts_layer
    @test_pnt_group.entities.add_cpoint(@position)
    @test_pnt_group.set_attribute "LSS Chronolux","transparent",true #mark all group as transparent to "ignore" while performing the Lss_Raytest
    
    point = Geom::Point3d.new(@position)
    transform = Geom::Transformation.new(point)
    entities = @test_pnt_group.entities
    path = Sketchup.find_support_file "pnt_mark.skp","Plugins/Chronolux/"
    definitions = @model.definitions
    componentdefinition = definitions.load path
    @model.start_operation($chronoluxStrings.GetString("Add Test Point..."))
    instance = entities.add_instance componentdefinition, transform
    instance.layer=@chrono_layers.test_pnts_layer
	# if some problems with a file of component instance, which represents a test point itself occured,
	# then place construction point instead of component instance
    if (instance)
      
    else
      c_pnt=entities.add_cpoint(@position)
      c_pnt.layer=@chrono_layers.test_pnts_layer
    end
    
    txt_pos=Geom::Point3d.new(@position)
    pos_offset=Geom::Vector3d.new(10,10,10)
    txt_pos.offset!(pos_offset)
    pnt_no_text=@test_pnt_group.entities.add_text(@pnt_no.to_s, txt_pos)
    pnt_no_text.layer=@chrono_layers.test_pnts_layer
    self.write_test_pnt_props
    @model.commit_operation
    value = @model.set_attribute "LSS Chronolux Settings", "pnt_no", @pnt_no+1
    
  end
  
  # Calculate insolation duration
  def calculate_insolation_duration(draw_rays) # draw_rays flag determins show or hide calculation process
    self.clear_old_intervals
    @ins_intervals_array=Array.new
    @sun_vec_prev=Geom::Vector3d.new
    @sun_rays_group=@entities.add_group
    @sun_rays_group.set_attribute "LSS Chronolux", "transparent",true #mark all group as transparent to "ignore" while performing the Lss_Raytest
    @sun_rays_group.name="LSS Sun Rays Group"
    @sun_rays_group.layer=@chrono_layers.rays_layer
    @shadowinfo = @model.shadow_info
    @curr_time=@start_time
    @calc_in_progress=true
    @intersect_previous=true
    if draw_rays
      chk_duration=@end_time-@start_time
      prgr_bar=Lss_Progr_Bar.new(chk_duration,"|","_",2)
      time_iterate_timer_id=UI.start_timer(0.01,true){
        ready_duraration=@curr_time-@start_time
        hr=@curr_time.utc.hour.to_s; hr="0"+hr if hr.length==1
        mn=@curr_time.utc.min.to_s; mn="0"+mn if mn.length==1
        time_str="#{hr}:#{mn}"
        prgr_bar.update(ready_duraration)
        Sketchup.status_text = "#{$chronoluxStrings.GetString("Rays calculation is in progress:")} #{time_str} #{prgr_bar.progr_string}"
        @shadowinfo["ShadowTime"]=@curr_time
        sun_dir=@shadowinfo["SunDirection"]
        @sun_vec=Geom::Vector3d.new(sun_dir)
        self.one_calc_step
        if @curr_time>=@end_time+@time_increment
          time_iterate_timer_id=UI.stop_timer(time_iterate_timer_id)
          self.draw_sun_rays_calc_results
          @calc_in_progress=false
		  Sketchup.status_text = ""
        end
        if @stop_calc
          time_iterate_timer_id=UI.stop_timer(time_iterate_timer_id)
          @calc_in_progress=false
          @stop_calc=false
		  Sketchup.status_text = ""
        end
      }
    else
      @calc_in_progress=true
      @lss_sun_scale=Lss_Sun_Scale.new
      @lss_sun_scale.read_sun_scale_from_model
      @sun_scale_times=Array.new(@lss_sun_scale.sun_scale_times)
      @sun_scale_dir_vecs=Array.new(@lss_sun_scale.sun_scale_dir_vecs)
      if @sun_scale_times.length==0
        @lss_sun_scale.generate_sun_scale
        @lss_sun_scale.write_sun_scale2model
        @sun_scale_times=Array.new(@lss_sun_scale.sun_scale_times)
        @sun_scale_dir_vecs=Array.new(@lss_sun_scale.sun_scale_dir_vecs)
      end
      ray_ind=0
      prgr_bar=Lss_Progr_Bar.new(@sun_scale_times.length,"|","_",2)
      #solve the problem when the end time is not exactly matches the last time from sun_scale!!!
      while ray_ind<@sun_scale_dir_vecs.length
        hr=@curr_time.utc.hour.to_s; hr="0"+hr if hr.length==1
        mn=@curr_time.utc.min.to_s; mn="0"+mn if mn.length==1
        time_str="#{hr}:#{mn}"
        prgr_bar.update(ray_ind)
        Sketchup.status_text = "#{$chronoluxStrings.GetString("Rays calculation is in progress:")} #{time_str} #{prgr_bar.progr_string}"
        @sun_vec=Geom::Vector3d.new(@sun_scale_dir_vecs[ray_ind])
        self.one_calc_step
        ray_ind+=1
      end
      self.one_calc_step #it is necessary to understand why it works such a way later...
      self.draw_sun_rays_calc_results
      Sketchup.status_text = ""
      @calc_in_progress=false
    end
  end
  
  # Perform one insolation calculation step (test insolation at a certain time @curr_time)
  def one_calc_step
    @sun_vec.length=@short_ray_len
    @pnt1=Geom::Point3d.new(@position)
    sun_ray=[@pnt1, @sun_vec]
    intersect_res=Lss_Raytest.new(sun_ray).test
    @curr_time=@end_time if @curr_time>@end_time
    if intersect_res.nil?
      if not (@intersect_previous.nil?)
        interval_bound=Array.new
        interval_bound+=[@curr_time]
        interval_bound+=[@sun_vec]
        @ins_intervals_array+=[interval_bound]
        self.add_short_sun_ray
      else
        if not (@curr_time==@end_time)
          self.add_short_sun_ray
        else
          interval_bound=Array.new
          interval_bound+=[@curr_time]
          interval_bound+=[@sun_vec]
          @ins_intervals_array+=[interval_bound]
          self.add_short_sun_ray
        end
      end
    elsif @intersect_previous.nil?
      @sun_vec=Geom::Vector3d.new(@sun_vec_prev)
      interval_bound=Array.new
      interval_bound+=[@curr_time]
      interval_bound+=[@sun_vec]
      @ins_intervals_array+=[interval_bound]
    end
    @intersect_previous=Lss_Raytest.new(sun_ray).test
    @sun_vec_prev=Geom::Vector3d.new(@sun_vec)
    @curr_time+=@time_increment
  end
  
  # Draw insolation calculation results
  def draw_sun_rays_calc_results
    @model.start_operation($chronoluxStrings.GetString("Draw Rays Cluster"))
    self.draw_insolated_sector
    self.draw_long_rays
    self.put_duration_texts
	
	# Store "key" attributes (test point number and calculation time) to associate a group, which represents
	# test point, with a group, which represents calculation results at this point. It is necessary for
	# refreshing of calculation results (erasing old calculation results and generating new ones).
    @sun_rays_calc_time=Time.now
    @test_pnt_group.set_attribute "LSS Test Point Properties", "total_ins_duration", @total_ins_duration
    @test_pnt_group.set_attribute "LSS Test Point Properties", "sun_rays_calc_time", @sun_rays_calc_time
    @is_calculated=true
    self.write_key_fields(@sun_rays_group,"rays")
    @model.commit_operation
  end
  
  # Write "key" attributes to a given group, which represent calculation results
  def write_key_fields(grp, grp_type)
    case grp_type
      when "rays"
      @test_pnt_group.set_attribute "LSS Test Point Properties", "sun_rays_calc_time", @sun_rays_calc_time
      grp.set_attribute "LSS Test Point Properties", "pnt_no", @pnt_no
      grp.set_attribute "LSS Test Point Properties", "sun_rays_calc_time", @sun_rays_calc_time
      when "svf"
      @test_pnt_group.set_attribute "LSS Test Point Properties", "svf_calc_time", @svf_calc_time
      grp.set_attribute "LSS Test Point Properties", "pnt_no", @pnt_no
      grp.set_attribute "LSS Test Point Properties", "svf_calc_time", @svf_calc_time
      when "wvf"
      @test_pnt_group.set_attribute "LSS Test Point Properties", "wvf_calc_time", @wvf_calc_time
      grp.set_attribute "LSS Test Point Properties", "pnt_no", @pnt_no
      grp.set_attribute "LSS Test Point Properties", "wvf_calc_time", @wvf_calc_time
    end
  end
  
  # Draw a surface, which represents a sector of insolation presence
  def draw_insolated_sector
    sector_pnts_arr=Array.new(@ray_end_pnt_array)
	# Patcth to solve encoding issue
	rays_str=$chronoluxStrings.GetString("Rays")
    @model.set_attribute "LSS Chronolux Settings", "rays_str", rays_str
    rays_str=@model.get_attribute "LSS Chronolux Settings", "rays_str"
    # End of Patcth to solve encoding issue
    if @ins_sector_type==rays_str
      sector_pnts_arr.each{|pt| 
        offset_vec=@position.vector_to(pt)
        offset_vec.length=@ray_offset
        pt1=@position.offset(offset_vec)
        edg=@sun_rays_group.entities.add_line(pt1,pt)
        edg.layer=@chrono_layers.rays_layer
      }
    else
      materials=Sketchup.active_model.materials
      sector_mat=nil
      materials.each{|mat|
        if mat.name=="LSS Sector Material"
          sector_mat=mat
        end
      }
      if sector_mat.nil?
        sector_mat=materials.add("LSS Sector Material")
        sector_mat.color=[100,100,100]
        sector_mat.alpha=0.7
      end
      ray_ind=0
      sectors_mesh=Geom::PolygonMesh.new
      while ray_ind<sector_pnts_arr.length-1
        curr_time1=@ray_time_array[ray_ind]
        curr_time2=@ray_time_array[ray_ind+1]
        time_delta=curr_time2-curr_time1
        pt1=Geom::Point3d.new(sector_pnts_arr[ray_ind])
        offset_vec=@position.vector_to(pt1)
        offset_vec.length=@ray_offset
        pt2=@position.offset(offset_vec)
        
        pt3=Geom::Point3d.new(sector_pnts_arr[ray_ind+1])
        offset_vec=@position.vector_to(pt3)
        offset_vec.length=@ray_offset
        pt4=@position.offset(offset_vec)
        
        sectors_mesh.add_point(pt1); sectors_mesh.add_point(pt2); sectors_mesh.add_point(pt3); sectors_mesh.add_point(pt4)
        if time_delta<=@time_increment #could be less at the last step, when @end_time is not exactly matches @time_increment
          sectors_mesh.add_polygon(pt1,pt2,pt3)
          sectors_mesh.add_polygon(pt3,pt2,pt4)
        end
        ray_ind+=1
      end
      @sun_rays_group.entities.add_faces_from_mesh(sectors_mesh, 12, sector_mat, sector_mat)
    end
  end
  
  # Draw insolation intervals boundary rays (to accent the start and the end of the interval when insolation is present)
  def draw_long_rays
    @ins_intervals_array.each{|ins_int|
      curr_time=ins_int[0]
      sun_vec=ins_int[1]
      self.add_long_sun_ray(curr_time, sun_vec)
    }
  end
  
  # Draw intervals duration texts
  def put_duration_texts
    int_bound_no=0
    while int_bound_no<@ins_intervals_array.length-1
      int_bound_st=Array.new(@ins_intervals_array[int_bound_no])
      int_bound_end=Array.new(@ins_intervals_array[int_bound_no+1])
      int_duration1=Time.at(int_bound_end[0]-int_bound_st[0])
      int_duration=int_duration1.utc
      @total_ins_duration+=int_duration.to_f
      txt_pos_offset_vec=Geom.linear_combination(0.5, int_bound_st[1], 0.5, int_bound_end[1])
      txt_pos_offset_vec.length=1.1*@short_ray_len
      txt_pos=@pnt1.offset(txt_pos_offset_vec)
      h=$chronoluxStrings.GetString("h")
      m=$chronoluxStrings.GetString("m")
      hr=int_duration.hour.to_s; hr="0"+hr if hr.length==1
      mn=int_duration.min.to_s; mn="0"+mn if mn.length==1
      duration_str="#{hr}#{h}#{mn}#{m}"
      duration_text=@sun_rays_group.entities.add_text(duration_str, txt_pos)
      duration_text.layer=@chrono_layers.rays_layer
      int_bound_no+=2
      ins_interval_no=int_bound_no/2
      ins_int_attr_name="ins_interval "+ins_interval_no.to_s
      splitter_str=@global_settings.splitter_str
      interval_str=ins_interval_no.to_s+splitter_str
      hr=int_bound_st[0].hour.to_s; hr="0"+hr if hr.length==1
      mn=int_bound_st[0].min.to_s; mn="0"+mn if mn.length==1
      interval_str+=hr+":"+mn+splitter_str
      hr=int_bound_end[0].hour.to_s; hr="0"+hr if hr.length==1
      mn=int_bound_end[0].min.to_s; mn="0"+mn if mn.length==1
      interval_str+=hr+":"+mn+splitter_str
      interval_str+=duration_str
      @test_pnt_group.set_attribute "LSS Test Point Properties", ins_int_attr_name, interval_str
    end
  end
  
  # Draw one short ray, which indicates insolation presence
  def add_short_sun_ray
    @sun_vec.length=@short_ray_len
    pnt2=@pnt1.offset(@sun_vec)
    @ray_end_pnt_array+=[pnt2]
    @ray_time_array+=[@curr_time]
  end
  
  # Draw long sun ray, which indicates insolation interval boundary (start or end of insolation interval)
  def add_long_sun_ray(curr_time, sun_vec)
    sun_vec.length=@long_ray_len
    pnt2=@pnt1.offset(sun_vec)
    edg=@sun_rays_group.entities.add_line(@pnt1,pnt2)
    edg.layer=@chrono_layers.rays_layer
    txt_pos=Geom::Point3d.new(pnt2)
    pos_offset=Geom::Vector3d.new(1,1,1)
    txt_pos.offset!(pos_offset)
    real_time=curr_time.utc
    hr=real_time.hour.to_s; hr="0"+hr if hr.length==1
    mn=real_time.min.to_s; mn="0"+mn if mn.length==1
    time_str="#{hr}:#{mn}"
    time_text=@sun_rays_group.entities.add_text(time_str, txt_pos)
    time_text.layer=@chrono_layers.rays_layer
  end
  
  # Calculate SVF and draw calculation results
  def calc_n_draw_svf(position, svf_sphere_r, svf_steps_cnt, show_process)
    @svf_shadow_mesh=Geom::PolygonMesh.new
    
    lss_svf_sphere=Lss_SVF.new(position, svf_sphere_r, svf_steps_cnt)
    lss_svf_sphere.build_rays_array
    lss_svf_sphere.calc_hemisphere_area
    @hemisphere_area=lss_svf_sphere.hemisphere_area
    lss_svf_sphere.perform_ray_test
    @svf_sphere_pnts=lss_svf_sphere.sphere_pnts
    @svf_result_pnts=lss_svf_sphere.results_array
    @segm_corners=lss_svf_sphere.segm_corners
    @svf_shadowed_area=0
    @prgr_bar=Lss_Progr_Bar.new(@svf_sphere_pnts.length,"|","_",2)
    if show_process
      @calc_in_progress=true
      ray_ind=0
      svf_timer_id=UI.start_timer(0.01,true){
        self.svf_one_step(ray_ind)
        ray_ind+=1
        if ray_ind>=@svf_result_pnts.length-1
          svf_timer_id=UI.stop_timer(svf_timer_id)
          @calc_in_progress=false
          self.svf_final_step
        end
        if @stop_calc
          svf_timer_id=UI.stop_timer(svf_timer_id)
          @calc_in_progress=false
          @stop_calc=false
        end
      }
    else
      @calc_in_progress=true
      @svf_result_pnts.each_index{|ray_ind|
        self.svf_one_step(ray_ind)
      }
      @calc_in_progress=false
      self.svf_final_step
    end
  end
  
  # Perform the final step of SVF calculation and generate a group, which represents SVF calculation results
  def svf_final_step
    @svf_group=@entities.add_group
    @svf_group.set_attribute "LSS Chronolux", "transparent",true #mark all group as transparent to "ignore" while performing the Lss_Raytest
    @svf_group.name="LSS SVF Group"
    @svf_group.layer=@chrono_layers.svf_layer
    @svf_grid_group=@svf_group.entities.add_group
    @svf_grid_group.layer=@chrono_layers.svf_layer
    
    materials=Sketchup.active_model.materials
    @svf_mat=nil
    materials.each{|mat|
      if mat.name=="LSS SVF Shadow Material"
        @svf_mat=mat
      end
    }
    if @svf_mat.nil?
      @svf_mat=materials.add("LSS SVF Shadow Material")
      @svf_mat.color=[100,100,100]
      @svf_mat.alpha=0.7
    end
    @svf_clear_area=@hemisphere_area-@svf_shadowed_area
    @svf=(10000.0*@svf_clear_area/@hemisphere_area).round/100.0
    svf_str="#{$chronoluxStrings.GetString("SVF")}+#{@svf}%"
    
    @model.start_operation($chronoluxStrings.GetString("Calculate and Draw SVF"))
    @svf_sphere_pnts.each_index{|ray_ind|
      if ray_ind<@svf_sphere_pnts.length-@svf_steps_cnt
        chk_mod=(ray_ind+@svf_steps_cnt+1).divmod(@svf_steps_cnt)[1]
        if chk_mod!=0
          segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
          segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind+1])
          segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
          segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt+1])
        else
          segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
          segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind-@svf_steps_cnt+1])
          segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
          segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+1])
        end
        edg=@svf_grid_group.entities.add_line(segm_pnt1, segm_pnt2)
        edg.layer=@chrono_layers.svf_layer if edg
        edg=@svf_grid_group.entities.add_line(segm_pnt1, segm_pnt3)
        edg.layer=@chrono_layers.svf_layer if edg
        edg=@svf_grid_group.entities.add_line(segm_pnt3, segm_pnt4)
        edg.layer=@chrono_layers.svf_layer if edg
        edg=@svf_grid_group.entities.add_line(segm_pnt4, segm_pnt2)
        edg.layer=@chrono_layers.svf_layer if edg
      end
    }
    @svf_group.entities.add_faces_from_mesh(@svf_shadow_mesh, 12, @svf_mat, @svf_mat)
    txt_pt=@svf_sphere_pnts[0]+[10,10,10]
    txt=@svf_group.entities.add_text(svf_str,txt_pt)
    txt_layer=@chrono_layers.svf_layer
    
	# Store "key" attributes (test point number and calculation time) to associate a group, which represents
	# test point, with a group, which represents calculation results at this point. It is necessary for
	# refreshing of calculation results (erasing old calculation results and generating new ones).
    @svf_calc_time=Time.now
    @test_pnt_group.set_attribute "LSS Test Point Properties", "svf", @svf
    @test_pnt_group.set_attribute "LSS Test Point Properties", "svf_calc_time", @svf_calc_time
    @is_calculated=true
    
    self.write_key_fields(@svf_group,"svf")
    @model.commit_operation
    result = Sketchup.status_text = "#{$chronoluxStrings.GetString("SVF")}=#{@svf}%"
  end
  
  # Perform one step of SVF calculation
  def svf_one_step(ray_ind)
    @prgr_bar.update(ray_ind)
    result = Sketchup.status_text = "#{$chronoluxStrings.GetString("Calculating SVF in progress:")} #{@prgr_bar.progr_string}"
    sph_pnt=Geom::Point3d.new(@svf_sphere_pnts[ray_ind])
    begin
      outer_pnt=Geom::Point3d.new(@svf_result_pnts[ray_ind])
    rescue
      outer_pnt=nil
    end
    if ray_ind<@svf_sphere_pnts.length-@svf_steps_cnt
      chk_mod=(ray_ind+@svf_steps_cnt+1).divmod(@svf_steps_cnt)[1]
      if chk_mod!=0
        segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
        segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind+1])
        segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
        segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt+1])
      else
        segm_pnt1=Geom::Point3d.new(@segm_corners[ray_ind])
        segm_pnt2=Geom::Point3d.new(@segm_corners[ray_ind-@svf_steps_cnt+1])
        segm_pnt3=Geom::Point3d.new(@segm_corners[ray_ind+@svf_steps_cnt])
        segm_pnt4=Geom::Point3d.new(@segm_corners[ray_ind+1])
      end
      if outer_pnt
        @svf_shadow_mesh.add_point(segm_pnt1);@svf_shadow_mesh.add_point(segm_pnt2)
        @svf_shadow_mesh.add_point(segm_pnt3);@svf_shadow_mesh.add_point(segm_pnt4)
        @svf_shadow_mesh.add_polygon(segm_pnt1,segm_pnt2,segm_pnt3)
		area1=Lss_Geom.new.calc_triangle_area(segm_pnt1,segm_pnt2,segm_pnt3)
		@svf_shadow_mesh.add_polygon(segm_pnt2,segm_pnt4,segm_pnt3)
		area2=Lss_Geom.new.calc_triangle_area(segm_pnt2,segm_pnt4,segm_pnt3)
        @svf_shadowed_area+=area1+area2
      end
    end
  end
end #class Lss_Test_Point